@@ -21,7 +21,9 @@
 #include "config.h"
 #include "wine/port.h"
 
+#include "initguid.h"
 #include "d3dcompiler_private.h"
+#include "winternl.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3dcompiler);
 
@@ -665,6 +667,14 @@ static UINT STDMETHODCALLTYPE d3dcompiler_shader_reflection_GetThreadGroupSize(
     return 0;
 }
 
+static UINT64 STDMETHODCALLTYPE d3dcompiler_shader_reflection_GetRequiresFlags(
+        ID3D11ShaderReflection *iface)
+{
+    FIXME("iface %p stub!\n", iface);
+
+    return 0;
+}
+
 static const struct ID3D11ShaderReflectionVtbl d3dcompiler_shader_reflection_vtbl =
 {
     /* IUnknown methods */
@@ -690,6 +700,7 @@ static const struct ID3D11ShaderReflectionVtbl d3dcompiler_shader_reflection_vtb
     d3dcompiler_shader_reflection_GetNumInterfaceSlots,
     d3dcompiler_shader_reflection_GetMinFeatureLevel,
     d3dcompiler_shader_reflection_GetThreadGroupSize,
+    d3dcompiler_shader_reflection_GetRequiresFlags,
 };
 
 /* ID3D11ShaderReflectionConstantBuffer methods */
@@ -1617,10 +1628,14 @@ static HRESULT d3dcompiler_parse_signature(struct d3dcompiler_shader_signature *
 
             if (d[i].Register == 0xffffffff)
             {
-                if (!strcasecmp(d[i].SemanticName, "sv_depth")) d[i].SystemValueType = D3D_NAME_DEPTH;
-                if (!strcasecmp(d[i].SemanticName, "sv_coverage")) d[i].SystemValueType = D3D_NAME_COVERAGE;
-                if (!strcasecmp(d[i].SemanticName, "sv_depthgreaterequal")) d[i].SystemValueType = D3D_NAME_DEPTH_GREATER_EQUAL;
-                if (!strcasecmp(d[i].SemanticName, "sv_depthlessequal")) d[i].SystemValueType = D3D_NAME_DEPTH_LESS_EQUAL;
+                if (!_strnicmp(d[i].SemanticName, "sv_depth", -1))
+                    d[i].SystemValueType = D3D_NAME_DEPTH;
+                else if (!_strnicmp(d[i].SemanticName, "sv_coverage", -1))
+                    d[i].SystemValueType = D3D_NAME_COVERAGE;
+                else if (!_strnicmp(d[i].SemanticName, "sv_depthgreaterequal", -1))
+                    d[i].SystemValueType = D3D_NAME_DEPTH_GREATER_EQUAL;
+                else if (!_strnicmp(d[i].SemanticName, "sv_depthlessequal", -1))
+                    d[i].SystemValueType = D3D_NAME_DEPTH_LESS_EQUAL;
             }
             else
             {
