@@ -90,26 +90,24 @@ NTSTATUS HID_LinkDevice(DEVICE_OBJECT *device)
 
     RtlInitUnicodeString( &nameW, ext->device_name);
 
-    devinfo = SetupDiGetClassDevsW(&GUID_DEVCLASS_HIDCLASS, NULL, NULL, DIGCF_DEVICEINTERFACE);
-    if (!devinfo)
+    devinfo = SetupDiCreateDeviceInfoList(&GUID_DEVCLASS_HIDCLASS, NULL);
+    if (devinfo == INVALID_HANDLE_VALUE)
     {
         FIXME( "failed to get ClassDevs %x\n", GetLastError());
         return STATUS_UNSUCCESSFUL;
     }
     Data.cbSize = sizeof(Data);
-    if (!SetupDiCreateDeviceInfoW(devinfo, ext->instance_id, &GUID_DEVCLASS_HIDCLASS, NULL, NULL, DICD_INHERIT_CLASSDRVS, &Data))
+    if (SetupDiCreateDeviceInfoW(devinfo, ext->instance_id, &GUID_DEVCLASS_HIDCLASS, NULL, NULL, DICD_INHERIT_CLASSDRVS, &Data))
     {
-        if (GetLastError() == ERROR_DEVINST_ALREADY_EXISTS)
+        if (!SetupDiRegisterDeviceInfo(devinfo, &Data, 0, NULL, NULL, NULL))
         {
-            SetupDiDestroyDeviceInfoList(devinfo);
-            return STATUS_SUCCESS;
+            FIXME( "failed to register device info %x\n", GetLastError());
+            goto error;
         }
-        FIXME( "failed to Create Device Info %x\n", GetLastError());
-        goto error;
     }
-    if (!SetupDiRegisterDeviceInfo( devinfo, &Data, 0, NULL, NULL, NULL ))
+    else if (GetLastError() != ERROR_DEVINST_ALREADY_EXISTS)
     {
-        FIXME( "failed to Register Device Info %x\n", GetLastError());
+        FIXME( "failed to create device info %x\n", GetLastError());
         goto error;
     }
     SetupDiDestroyDeviceInfoList(devinfo);
@@ -128,10 +126,55 @@ error:
     return STATUS_UNSUCCESSFUL;
 }
 
+static IRP *pop_irp_from_queue(BASE_DEVICE_EXTENSION *ext)
+{
+    LIST_ENTRY *entry;
+    KIRQL old_irql;
+    IRP *irp = NULL;
+
+    KeAcquireSpinLock(&ext->irp_queue_lock, &old_irql);
+
+    while (!irp && (entry = RemoveHeadList(&ext->irp_queue)) != &ext->irp_queue)
+    {
+        irp = CONTAINING_RECORD(entry, IRP, Tail.Overlay.s.ListEntry);
+        if (!IoSetCancelRoutine(irp, NULL))
+        {
+            /* cancel routine is already cleared, meaning that it was called. let it handle completion. */
+            InitializeListHead(&irp->Tail.Overlay.s.ListEntry);
+            irp = NULL;
+        }
+    }
+
+    KeReleaseSpinLock(&ext->irp_queue_lock, old_irql);
+    return irp;
+}
+
+static void WINAPI read_cancel_routine(DEVICE_OBJECT *device, IRP *irp)
+{
+    BASE_DEVICE_EXTENSION *ext;
+    KIRQL old_irql;
+
+    TRACE("cancel %p IRP on device %p\n", irp, device);
+
+    ext = device->DeviceExtension;
+
+    IoReleaseCancelSpinLock(irp->CancelIrql);
+
+    KeAcquireSpinLock(&ext->irp_queue_lock, &old_irql);
+
+    RemoveEntryList(&irp->Tail.Overlay.s.ListEntry);
+
+    KeReleaseSpinLock(&ext->irp_queue_lock, old_irql);
+
+    irp->IoStatus.u.Status = STATUS_CANCELLED;
+    irp->IoStatus.Information = 0;
+    IoCompleteRequest(irp, IO_NO_INCREMENT);
+}
+
+
 void HID_DeleteDevice(HID_MINIDRIVER_REGISTRATION *driver, DEVICE_OBJECT *device)
 {
     BASE_DEVICE_EXTENSION *ext;
-    LIST_ENTRY *entry;
     IRP *irp;
 
     ext = device->DeviceExtension;
@@ -147,13 +190,10 @@ void HID_DeleteDevice(HID_MINIDRIVER_REGISTRATION *driver, DEVICE_OBJECT *device
     if (ext->ring_buffer)
         RingBuffer_Destroy(ext->ring_buffer);
 
-    entry = RemoveHeadList(&ext->irp_queue);
-    while(entry != &ext->irp_queue)
+    while((irp = pop_irp_from_queue(ext)))
     {
-        irp = CONTAINING_RECORD(entry, IRP, Tail.Overlay.s.ListEntry);
         irp->IoStatus.u.Status = STATUS_DEVICE_REMOVED;
         IoCompleteRequest(irp, IO_NO_INCREMENT);
-        entry = RemoveHeadList(&ext->irp_queue);
     }
 
     TRACE("Delete device(%p) %s\n", device, debugstr_w(ext->device_name));
@@ -191,7 +231,6 @@ static NTSTATUS copy_packet_into_buffer(HID_XFER_PACKET *packet, BYTE* buffer, U
 
 static void HID_Device_processQueue(DEVICE_OBJECT *device)
 {
-    LIST_ENTRY *entry;
     IRP *irp;
     BASE_DEVICE_EXTENSION *ext = device->DeviceExtension;
     UINT buffer_size = RingBuffer_GetBufferSize(ext->ring_buffer);
@@ -199,11 +238,9 @@ static void HID_Device_processQueue(DEVICE_OBJECT *device)
 
     packet = HeapAlloc(GetProcessHeap(), 0, buffer_size);
 
-    entry = RemoveHeadList(&ext->irp_queue);
-    while(entry != &ext->irp_queue)
+    while((irp = pop_irp_from_queue(ext)))
     {
         int ptr;
-        irp = CONTAINING_RECORD(entry, IRP, Tail.Overlay.s.ListEntry);
         ptr = PtrToUlong( irp->Tail.Overlay.OriginalFileObject->FsContext );
 
         RingBuffer_Read(ext->ring_buffer, ptr, packet, &buffer_size);
@@ -224,7 +261,6 @@ static void HID_Device_processQueue(DEVICE_OBJECT *device)
             irp->IoStatus.u.Status = STATUS_UNSUCCESSFUL;
         }
         IoCompleteRequest( irp, IO_NO_INCREMENT );
-        entry = RemoveHeadList(&ext->irp_queue);
     }
     HeapFree(GetProcessHeap(), 0, packet);
 }
@@ -658,8 +694,24 @@ NTSTATUS WINAPI HID_Device_read(DEVICE_OBJECT *device, IRP *irp)
         BASE_DEVICE_EXTENSION *extension = device->DeviceExtension;
         if (extension->poll_interval)
         {
+            KIRQL old_irql;
             TRACE_(hid_report)("Queue irp\n");
+
+            KeAcquireSpinLock(&ext->irp_queue_lock, &old_irql);
+
+            IoSetCancelRoutine(irp, read_cancel_routine);
+            if (irp->Cancel && !IoSetCancelRoutine(irp, NULL))
+            {
+                /* IRP was canceled before we set cancel routine */
+                InitializeListHead(&irp->Tail.Overlay.s.ListEntry);
+                KeReleaseSpinLock(&ext->irp_queue_lock, old_irql);
+                return STATUS_CANCELLED;
+            }
+
             InsertTailList(&ext->irp_queue, &irp->Tail.Overlay.s.ListEntry);
+            IoMarkIrpPending(irp);
+
+            KeReleaseSpinLock(&ext->irp_queue_lock, old_irql);
             rc = STATUS_PENDING;
         }
         else
