@@ -24,6 +24,7 @@
 #include <stdlib.h>
 
 #include "ntdll_test.h"
+#include "winnls.h"
 
 
 /* Function ptrs for ntdll calls */
@@ -61,6 +62,9 @@ static void     (__cdecl *p_qsort)(void *,size_t,size_t, int(__cdecl *compar)(co
 static void*    (__cdecl *p_bsearch)(void *,void*,size_t,size_t, int(__cdecl *compar)(const void *, const void *) );
 static int      (WINAPIV *p__snprintf)(char *, size_t, const char *, ...);
 
+static int      (__cdecl *p_tolower)(int);
+static int      (__cdecl *p_toupper)(int);
+static int      (__cdecl *p__strnicmp)(LPCSTR,LPCSTR,size_t);
 
 static void InitFunctionPtrs(void)
 {
@@ -99,6 +103,10 @@ static void InitFunctionPtrs(void)
 	p_bsearch= (void *)GetProcAddress(hntdll, "bsearch");
 
         p__snprintf = (void *)GetProcAddress(hntdll, "_snprintf");
+
+        p_tolower = (void *)GetProcAddress(hntdll, "tolower");
+        p_toupper = (void *)GetProcAddress(hntdll, "toupper");
+        p__strnicmp = (void *)GetProcAddress(hntdll, "_strnicmp");
     } /* if */
 }
 
@@ -1327,6 +1335,77 @@ static void test__snprintf(void)
     ok(!strcmp(buffer, teststring), "_snprintf returned buffer '%s', expected '%s'.\n", buffer, teststring);
 }
 
+static void test_tolower(void)
+{
+    int i, ret, exp_ret;
+
+    if (!GetProcAddress(GetModuleHandleA("ntdll"), "NtRemoveIoCompletionEx"))
+    {
+        win_skip("tolower tests\n");
+        return;
+    }
+
+    ok(p_tolower != NULL, "tolower is not available\n");
+
+    for (i = -512; i < 512; i++)
+    {
+        exp_ret = (char)i >= 'A' && (char)i <= 'Z' ? i - 'A' + 'a' : i;
+        ret = p_tolower(i);
+        ok(ret == exp_ret, "tolower(%d) = %d\n", i, ret);
+    }
+}
+
+static void test_toupper(void)
+{
+
+    int i, ret, exp_ret;
+    char str[2], *p;
+    WCHAR wc;
+
+    ok(p_toupper != NULL, "toupper is not available\n");
+
+    for (i = -512; i < 0xffff; i++)
+    {
+        str[0] = i;
+        str[1] = i >> 8;
+        p = str;
+        wc = RtlAnsiCharToUnicodeChar( &p );
+        wc = RtlUpcaseUnicodeChar( wc );
+        ret = WideCharToMultiByte( CP_ACP, 0, &wc, 1, str, 2, NULL, NULL );
+        ok(ret == 1 || ret == 2, "WideCharToMultiByte returned %d\n", ret);
+        if (ret == 2)
+            exp_ret = (unsigned char)str[1] + ((unsigned char)str[0] << 8);
+        else
+            exp_ret = (unsigned char)str[0];
+
+        ret = p_toupper(i);
+        ok(ret == exp_ret, "toupper(%x) = %x, expected %x\n", i, ret, exp_ret);
+    }
+}
+
+static void test__strnicmp(void)
+{
+    BOOL is_win64 = (sizeof(void *) > sizeof(int));
+    int ret;
+
+    ok(p__strnicmp != NULL, "_strnicmp is not available\n");
+
+    ret = p__strnicmp("a", "C", 1);
+    ok(ret == (is_win64 ? -2 : -1), "_strnicmp returned %d\n", ret);
+    ret = p__strnicmp("a", "c", 1);
+    ok(ret == (is_win64 ? -2 : -1), "_strnicmp returned %d\n", ret);
+    ret = p__strnicmp("C", "a", 1);
+    ok(ret == (is_win64 ? 2 : 1), "_strnicmp returned %d\n", ret);
+    ret = p__strnicmp("c", "a", 1);
+    ok(ret == (is_win64 ? 2 : 1), "_strnicmp returned %d\n", ret);
+    ret = p__strnicmp("ijk0", "IJK1", 3);
+    ok(!ret, "_strnicmp returned %d\n", ret);
+    ret = p__strnicmp("ijk0", "IJK1", 4);
+    ok(ret == -1, "_strnicmp returned %d\n", ret);
+    ret = p__strnicmp("ijk\0X", "IJK\0Y", 5);
+    ok(!ret, "_strnicmp returned %d\n", ret);
+}
+
 START_TEST(string)
 {
     InitFunctionPtrs();
@@ -1363,4 +1442,7 @@ START_TEST(string)
         test_bsearch();
     if (p__snprintf)
         test__snprintf();
+    test_tolower();
+    test_toupper();
+    test__strnicmp();
 }
