@@ -1223,7 +1223,13 @@ static const ISpecifyPropertyPagesVtbl SpecifyPropertyPagesVtbl = {
     SpecifyPropertyPages_GetPages
 };
 
-static HRESULT WINAPI AviMuxOut_AttemptConnection(BasePin *base,
+static HRESULT WINAPI AviMuxOut_CheckMediaType(BasePin *base, const AM_MEDIA_TYPE *amt)
+{
+    FIXME("(%p) stub\n", base);
+    return S_OK;
+}
+
+static HRESULT WINAPI AviMuxOut_AttemptConnection(BaseOutputPin *base,
         IPin *pReceivePin, const AM_MEDIA_TYPE *pmt)
 {
     PIN_DIRECTION dir;
@@ -1239,11 +1245,6 @@ static HRESULT WINAPI AviMuxOut_AttemptConnection(BasePin *base,
     return BaseOutputPinImpl_AttemptConnection(base, pReceivePin, pmt);
 }
 
-static LONG WINAPI AviMuxOut_GetMediaTypeVersion(BasePin *base)
-{
-    return 0;
-}
-
 static HRESULT WINAPI AviMuxOut_GetMediaType(BasePin *base, int iPosition, AM_MEDIA_TYPE *amt)
 {
     TRACE("(%p)->(%d %p)\n", base, iPosition, amt);
@@ -1299,11 +1300,10 @@ static HRESULT WINAPI AviMuxOut_BreakConnect(BaseOutputPin *base)
 
 static const BaseOutputPinFuncTable AviMuxOut_BaseOutputFuncTable = {
     {
-        NULL,
-        AviMuxOut_AttemptConnection,
-        AviMuxOut_GetMediaTypeVersion,
+        AviMuxOut_CheckMediaType,
         AviMuxOut_GetMediaType
     },
+    AviMuxOut_AttemptConnection,
     NULL,
     AviMuxOut_DecideAllocator,
     AviMuxOut_BreakConnect
@@ -1579,11 +1579,6 @@ static HRESULT WINAPI AviMuxIn_CheckMediaType(BasePin *base, const AM_MEDIA_TYPE
     return S_FALSE;
 }
 
-static LONG WINAPI AviMuxIn_GetMediaTypeVersion(BasePin *base)
-{
-    return 0;
-}
-
 static HRESULT WINAPI AviMuxIn_GetMediaType(BasePin *base, int iPosition, AM_MEDIA_TYPE *amt)
 {
     return S_FALSE;
@@ -1695,8 +1690,6 @@ static HRESULT WINAPI AviMuxIn_Receive(BaseInputPin *base, IMediaSample *pSample
 static const BaseInputPinFuncTable AviMuxIn_BaseInputFuncTable = {
     {
         AviMuxIn_CheckMediaType,
-        NULL,
-        AviMuxIn_GetMediaTypeVersion,
         AviMuxIn_GetMediaType
     },
     AviMuxIn_Receive
