@@ -180,26 +180,6 @@ static HRESULT WINAPI SmartTeeFilter_Run(IBaseFilter *iface, REFERENCE_TIME tSta
     return hr;
 }
 
-static HRESULT WINAPI SmartTeeFilter_FindPin(IBaseFilter *iface, LPCWSTR Id, IPin **ppPin)
-{
-    SmartTeeFilter *This = impl_from_IBaseFilter(iface);
-    TRACE("(%p)->(%s, %p)\n", This, debugstr_w(Id), ppPin);
-    if (lstrcmpW(Id, This->input->pin.pinInfo.achName) == 0) {
-        *ppPin = &This->input->pin.IPin_iface;
-        IPin_AddRef(*ppPin);
-        return S_OK;
-    } else if (lstrcmpW(Id, This->capture->pin.pinInfo.achName) == 0) {
-        *ppPin = &This->capture->pin.IPin_iface;
-        IPin_AddRef(*ppPin);
-        return S_OK;
-    } else if (lstrcmpW(Id, This->preview->pin.pinInfo.achName) == 0) {
-        *ppPin = &This->preview->pin.IPin_iface;
-        IPin_AddRef(*ppPin);
-        return S_OK;
-    }
-    return VFW_E_NOT_FOUND;
-}
-
 static const IBaseFilterVtbl SmartTeeFilterVtbl = {
     SmartTeeFilter_QueryInterface,
     SmartTeeFilter_AddRef,
@@ -212,7 +192,7 @@ static const IBaseFilterVtbl SmartTeeFilterVtbl = {
     BaseFilterImpl_SetSyncSource,
     BaseFilterImpl_GetSyncSource,
     BaseFilterImpl_EnumPins,
-    SmartTeeFilter_FindPin,
+    BaseFilterImpl_FindPin,
     BaseFilterImpl_QueryFilterInfo,
     BaseFilterImpl_JoinFilterGraph,
     BaseFilterImpl_QueryVendorInfo
@@ -299,11 +279,6 @@ static HRESULT WINAPI SmartTeeFilterInput_CheckMediaType(BasePin *base, const AM
     return S_OK;
 }
 
-static LONG WINAPI SmartTeeFilterInput_GetMediaTypeVersion(BasePin *base)
-{
-    return 0;
-}
-
 static HRESULT WINAPI SmartTeeFilterInput_GetMediaType(BasePin *base, int iPosition, AM_MEDIA_TYPE *amt)
 {
     SmartTeeFilter *This = impl_from_BasePin(base);
@@ -445,8 +420,6 @@ static HRESULT WINAPI SmartTeeFilterInput_Receive(BaseInputPin *base, IMediaSamp
 static const BaseInputPinFuncTable SmartTeeFilterInputFuncs = {
     {
         SmartTeeFilterInput_CheckMediaType,
-        NULL,
-        SmartTeeFilterInput_GetMediaTypeVersion,
         SmartTeeFilterInput_GetMediaType
     },
     SmartTeeFilterInput_Receive
@@ -499,11 +472,10 @@ static const IPinVtbl SmartTeeFilterCaptureVtbl = {
     BasePinImpl_NewSegment
 };
 
-static LONG WINAPI SmartTeeFilterCapture_GetMediaTypeVersion(BasePin *base)
+static HRESULT WINAPI SmartTeeFilterCapture_CheckMediaType(BasePin *base, const AM_MEDIA_TYPE *amt)
 {
-    SmartTeeFilter *This = impl_from_BasePin(base);
-    TRACE("(%p)\n", This);
-    return 0;
+    FIXME("(%p) stub\n", base);
+    return S_OK;
 }
 
 static HRESULT WINAPI SmartTeeFilterCapture_GetMediaType(BasePin *base, int iPosition, AM_MEDIA_TYPE *amt)
@@ -535,11 +507,10 @@ static HRESULT WINAPI SmartTeeFilterCapture_BreakConnect(BaseOutputPin *base)
 
 static const BaseOutputPinFuncTable SmartTeeFilterCaptureFuncs = {
     {
-        NULL,
-        BaseOutputPinImpl_AttemptConnection,
-        SmartTeeFilterCapture_GetMediaTypeVersion,
+        SmartTeeFilterCapture_CheckMediaType,
         SmartTeeFilterCapture_GetMediaType
     },
+    BaseOutputPinImpl_AttemptConnection,
     NULL,
     SmartTeeFilterCapture_DecideAllocator,
     SmartTeeFilterCapture_BreakConnect
@@ -592,11 +563,10 @@ static const IPinVtbl SmartTeeFilterPreviewVtbl = {
     BasePinImpl_NewSegment
 };
 
-static LONG WINAPI SmartTeeFilterPreview_GetMediaTypeVersion(BasePin *base)
+static HRESULT WINAPI SmartTeeFilterPreview_CheckMediaType(BasePin *base, const AM_MEDIA_TYPE *amt)
 {
-    SmartTeeFilter *This = impl_from_BasePin(base);
-    TRACE("(%p)\n", This);
-    return 0;
+    FIXME("(%p) stub\n", base);
+    return S_OK;
 }
 
 static HRESULT WINAPI SmartTeeFilterPreview_GetMediaType(BasePin *base, int iPosition, AM_MEDIA_TYPE *amt)
@@ -628,11 +598,10 @@ static HRESULT WINAPI SmartTeeFilterPreview_BreakConnect(BaseOutputPin *base)
 
 static const BaseOutputPinFuncTable SmartTeeFilterPreviewFuncs = {
     {
-        NULL,
-        BaseOutputPinImpl_AttemptConnection,
-        SmartTeeFilterPreview_GetMediaTypeVersion,
+        SmartTeeFilterPreview_CheckMediaType,
         SmartTeeFilterPreview_GetMediaType
     },
+    BaseOutputPinImpl_AttemptConnection,
     NULL,
     SmartTeeFilterPreview_DecideAllocator,
     SmartTeeFilterPreview_BreakConnect
