@@ -955,10 +955,8 @@ static HRESULT AVISplitter_InitializeStreams(AVISplitterImpl *This)
 
         for (x = 0; x < This->Parser.cStreams; ++x)
         {
-            if ((DWORD)This->streams[x].frames != This->streams[x].streamheader.dwLength)
-            {
-                FIXME("stream %u: frames found: %u, frames meant to be found: %u\n", x, (DWORD)This->streams[x].frames, This->streams[x].streamheader.dwLength);
-            }
+            if (This->streams[x].frames != This->streams[x].streamheader.dwLength)
+                FIXME("stream %u: frames found: %u, frames meant to be found: %u\n", x, This->streams[x].frames, This->streams[x].streamheader.dwLength);
         }
 
     }
@@ -1429,6 +1427,7 @@ static const IBaseFilterVtbl AVISplitterImpl_Vtbl =
 
 HRESULT AVISplitter_create(IUnknown * pUnkOuter, LPVOID * ppv)
 {
+    static const WCHAR sink_name[] = {'i','n','p','u','t',' ','p','i','n',0};
     HRESULT hr;
     AVISplitterImpl * This;
 
@@ -1445,7 +1444,11 @@ HRESULT AVISplitter_create(IUnknown * pUnkOuter, LPVOID * ppv)
     This->streams = NULL;
     This->oldindex = NULL;
 
-    hr = Parser_Create(&(This->Parser), &AVISplitterImpl_Vtbl, &CLSID_AviSplitter, AVISplitter_Sample, AVISplitter_QueryAccept, AVISplitter_InputPin_PreConnect, AVISplitter_Flush, AVISplitter_Disconnect, AVISplitter_first_request, AVISplitter_done_process, NULL, AVISplitter_seek, NULL);
+    hr = Parser_Create(&This->Parser, &AVISplitterImpl_Vtbl, &CLSID_AviSplitter,
+            sink_name, AVISplitter_Sample, AVISplitter_QueryAccept,
+            AVISplitter_InputPin_PreConnect, AVISplitter_Flush,
+            AVISplitter_Disconnect, AVISplitter_first_request,
+            AVISplitter_done_process, NULL, AVISplitter_seek, NULL);
 
     if (FAILED(hr))
         return hr;
