@@ -18,29 +18,13 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
-#include "config.h"
-#include <stdarg.h>
 
-#define COBJMACROS
-
-#include "windef.h"
-#include "winbase.h"
-#include "dshow.h"
-#include "amvideo.h"
-#include "strmif.h"
-#include "vfw.h"
-
-#include <assert.h>
-
-#include "wine/unicode.h"
-#include "wine/debug.h"
-#include "wine/strmbase.h"
 #include "strmbase_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(strmbase);
 
-static const WCHAR wcsInputPinName[] = {'i','n','p','u','t',' ','p','i','n',0};
-static const WCHAR wcsOutputPinName[] = {'o','u','t','p','u','t',' ','p','i','n',0};
+static const WCHAR wcsInputPinName[] = {'I','n',0};
+static const WCHAR wcsOutputPinName[] = {'O','u','t',0};
 
 static const IPinVtbl TransformFilter_InputPin_Vtbl;
 static const IPinVtbl TransformFilter_OutputPin_Vtbl;
@@ -144,22 +128,20 @@ static HRESULT WINAPI TransformFilter_Output_GetMediaType(BasePin *This, int iPo
     return S_OK;
 }
 
-static IPin* WINAPI TransformFilter_GetPin(BaseFilter *iface, int pos)
+static IPin *WINAPI TransformFilter_GetPin(BaseFilter *iface, int index)
 {
-    TransformFilter *This = impl_from_BaseFilter(iface);
+    TransformFilter *filter = impl_from_BaseFilter(iface);
 
-    if (pos >= This->npins || pos < 0)
+    if (index >= 2 || index < 0)
         return NULL;
 
-    IPin_AddRef(This->ppPins[pos]);
-    return This->ppPins[pos];
+    IPin_AddRef(filter->ppPins[index]);
+    return filter->ppPins[index];
 }
 
 static LONG WINAPI TransformFilter_GetPinCount(BaseFilter *iface)
 {
-    TransformFilter *This = impl_from_BaseFilter(iface);
-
-    return (This->npins+1);
+    return 2;
 }
 
 static const BaseFilterFuncTable tfBaseFuncTable = {
@@ -170,8 +152,6 @@ static const BaseFilterFuncTable tfBaseFuncTable = {
 static const BaseInputPinFuncTable tf_input_BaseInputFuncTable = {
     {
         TransformFilter_Input_CheckMediaType,
-        NULL,
-        BasePinImpl_GetMediaTypeVersion,
         BasePinImpl_GetMediaType
     },
     TransformFilter_Input_Receive
@@ -180,10 +160,9 @@ static const BaseInputPinFuncTable tf_input_BaseInputFuncTable = {
 static const BaseOutputPinFuncTable tf_output_BaseOutputFuncTable = {
     {
         TransformFilter_Output_CheckMediaType,
-        BaseOutputPinImpl_AttemptConnection,
-        BasePinImpl_GetMediaTypeVersion,
         TransformFilter_Output_GetMediaType
     },
+    BaseOutputPinImpl_AttemptConnection,
     TransformFilter_Output_DecideBufferSize,
     BaseOutputPinImpl_DecideAllocator,
     BaseOutputPinImpl_BreakConnect
@@ -203,7 +182,6 @@ static HRESULT TransformFilter_Init(const IBaseFilterVtbl *pVtbl, const CLSID* p
     /* pTransformFilter is already allocated */
     pTransformFilter->pFuncsTable = pFuncsTable;
     ZeroMemory(&pTransformFilter->pmt, sizeof(pTransformFilter->pmt));
-    pTransformFilter->npins = 2;
 
     pTransformFilter->ppPins = CoTaskMemAlloc(2 * sizeof(IPin *));
 
@@ -314,7 +292,7 @@ ULONG WINAPI TransformFilterImpl_Release(IBaseFilter * iface)
     {
         ULONG i;
 
-        for (i = 0; i < This->npins; i++)
+        for (i = 0; i < 2; i++)
         {
             IPin *pConnectedTo;
 
