@@ -260,6 +260,7 @@ static HWND *list_children_from_point( HWND hwnd, POINT pt )
             req->parent = wine_server_user_handle( hwnd );
             req->x = pt.x;
             req->y = pt.y;
+            req->dpi = get_thread_dpi();
             wine_server_set_reply( req, list, (size-1) * sizeof(user_handle_t) );
             if (!wine_server_call( req )) count = reply->count;
         }
@@ -289,6 +290,7 @@ HWND WINPOS_WindowFromPoint( HWND hwndScope, POINT pt, INT *hittest )
 {
     int i, res;
     HWND ret, *list;
+    POINT win_pt;
 
     if (!hwndScope) hwndScope = GetDesktopWindow();
 
@@ -303,11 +305,6 @@ HWND WINPOS_WindowFromPoint( HWND hwndScope, POINT pt, INT *hittest )
         LONG style = GetWindowLongW( list[i], GWL_STYLE );
 
         /* If window is minimized or disabled, return at once */
-        if (style & WS_MINIMIZE)
-        {
-            *hittest = HTCAPTION;
-            break;
-        }
         if (style & WS_DISABLED)
         {
             *hittest = HTERROR;
@@ -319,7 +316,8 @@ HWND WINPOS_WindowFromPoint( HWND hwndScope, POINT pt, INT *hittest )
             *hittest = HTCLIENT;
             break;
         }
-        res = SendMessageW( list[i], WM_NCHITTEST, 0, MAKELONG(pt.x,pt.y) );
+        win_pt = point_thread_to_win_dpi( list[i], pt );
+        res = SendMessageW( list[i], WM_NCHITTEST, 0, MAKELPARAM( win_pt.x, win_pt.y ));
         if (res != HTTRANSPARENT)
         {
             *hittest = res;  /* Found the window */
@@ -446,6 +444,7 @@ static BOOL WINPOS_GetWinOffset( HWND hwndFrom, HWND hwndTo, BOOL *mirrored, POI
                 }
             }
             if (wndPtr && wndPtr != WND_DESKTOP) WIN_ReleasePtr( wndPtr );
+            offset = point_win_to_thread_dpi( hwndFrom, offset );
         }
     }
 
@@ -460,15 +459,16 @@ static BOOL WINPOS_GetWinOffset( HWND hwndFrom, HWND hwndTo, BOOL *mirrored, POI
         if (wndPtr == WND_OTHER_PROCESS) goto other_process;
         if (wndPtr != WND_DESKTOP)
         {
+            POINT pt = { 0, 0 };
             if (wndPtr->dwExStyle & WS_EX_LAYOUTRTL)
             {
                 mirror_to = TRUE;
-                offset.x -= wndPtr->client_rect.right - wndPtr->client_rect.left;
+                pt.x += wndPtr->client_rect.right - wndPtr->client_rect.left;
             }
             while (wndPtr->parent)
             {
-                offset.x -= wndPtr->client_rect.left;
-                offset.y -= wndPtr->client_rect.top;
+                pt.x += wndPtr->client_rect.left;
+                pt.y += wndPtr->client_rect.top;
                 hwnd = wndPtr->parent;
                 WIN_ReleasePtr( wndPtr );
                 if (!(wndPtr = WIN_GetPtr( hwnd ))) break;
@@ -481,6 +481,9 @@ static BOOL WINPOS_GetWinOffset( HWND hwndFrom, HWND hwndTo, BOOL *mirrored, POI
                 }
             }
             if (wndPtr && wndPtr != WND_DESKTOP) WIN_ReleasePtr( wndPtr );
+            pt = point_win_to_thread_dpi( hwndTo, pt );
+            offset.x -= pt.x;
+            offset.y -= pt.y;
         }
     }
 
@@ -494,6 +497,7 @@ static BOOL WINPOS_GetWinOffset( HWND hwndFrom, HWND hwndTo, BOOL *mirrored, POI
     {
         req->from = wine_server_user_handle( hwndFrom );
         req->to   = wine_server_user_handle( hwndTo );
+        req->dpi  = get_thread_dpi();
         if ((ret = !wine_server_call_err( req )))
         {
             ret_offset->x = reply->x;
@@ -674,67 +678,6 @@ BOOL WINAPI MoveWindow( HWND hwnd, INT x, INT y, INT cx, INT cy,
 }
 
 
-/***********************************************************************
- *           WINPOS_RedrawIconTitle
- */
-BOOL WINPOS_RedrawIconTitle( HWND hWnd )
-{
-    HWND icon_title = 0;
-    WND *win = WIN_GetPtr( hWnd );
-
-    if (win && win != WND_OTHER_PROCESS && win != WND_DESKTOP)
-    {
-	icon_title = win->icon_title;
-        WIN_ReleasePtr( win );
-    }
-    if (!icon_title) return FALSE;
-    SendMessageW( icon_title, WM_SHOWWINDOW, TRUE, 0 );
-    InvalidateRect( icon_title, NULL, TRUE );
-    return TRUE;
-}
-
-/***********************************************************************
- *           WINPOS_ShowIconTitle
- */
-static void WINPOS_ShowIconTitle( HWND hwnd, BOOL bShow )
-{
-    WND *win = WIN_GetPtr( hwnd );
-    HWND title = 0;
-
-    TRACE("%p %i\n", hwnd, (bShow != 0) );
-
-    if (!win || win == WND_OTHER_PROCESS || win == WND_DESKTOP) return;
-    if (win->window_rect.left == -32000 || win->window_rect.top == -32000)
-    {
-        TRACE( "not showing title for hidden icon %p\n", hwnd );
-        bShow = FALSE;
-    }
-    else title = win->icon_title;
-    WIN_ReleasePtr( win );
-
-    if (bShow)
-    {
-        if (!title)
-        {
-            title = ICONTITLE_Create( hwnd );
-            if (!(win = WIN_GetPtr( hwnd )) || win == WND_OTHER_PROCESS)
-            {
-                DestroyWindow( title );
-                return;
-            }
-            win->icon_title = title;
-            WIN_ReleasePtr( win );
-        }
-        if (!IsWindowVisible(title))
-        {
-            SendMessageW( title, WM_SHOWWINDOW, TRUE, 0 );
-            SetWindowPos( title, 0, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE |
-                          SWP_NOACTIVATE | SWP_NOZORDER | SWP_SHOWWINDOW );
-        }
-    }
-    else if (title) ShowWindow( title, SW_HIDE );
-}
-
 /*******************************************************************
  *           WINPOS_GetMinMaxInfo
  *
@@ -742,6 +685,7 @@ static void WINPOS_ShowIconTitle( HWND hwnd, BOOL bShow )
  */
 MINMAXINFO WINPOS_GetMinMaxInfo( HWND hwnd )
 {
+    DPI_AWARENESS_CONTEXT context;
     MINMAXINFO MinMax;
     HMONITOR monitor;
     INT xinc, yinc;
@@ -751,6 +695,8 @@ MINMAXINFO WINPOS_GetMinMaxInfo( HWND hwnd )
     RECT rc;
     WND *win;
 
+    context = SetThreadDpiAwarenessContext( GetWindowDpiAwarenessContext( hwnd ));
+
     /* Compute default values */
 
     GetWindowRect(hwnd, &rc);
@@ -836,25 +782,87 @@ MINMAXINFO WINPOS_GetMinMaxInfo( HWND hwnd )
     MinMax.ptMaxTrackSize.y = max( MinMax.ptMaxTrackSize.y,
                                    MinMax.ptMinTrackSize.y );
 
+    SetThreadDpiAwarenessContext( context );
     return MinMax;
 }
 
+static POINT get_first_minimized_child_pos( const RECT *parent, const MINIMIZEDMETRICS *mm,
+                                            int width, int height )
+{
+    POINT ret;
 
-/***********************************************************************
- *           WINPOS_FindIconPos
- *
- * Find a suitable place for an iconic window.
- */
-static POINT WINPOS_FindIconPos( HWND hwnd, POINT pt )
+    if (mm->iArrange & ARW_STARTRIGHT)
+        ret.x = parent->right - mm->iHorzGap - width;
+    else
+        ret.x = parent->left + mm->iHorzGap;
+    if (mm->iArrange & ARW_STARTTOP)
+        ret.y = parent->top + mm->iVertGap;
+    else
+        ret.y = parent->bottom - mm->iVertGap - height;
+
+    return ret;
+}
+
+static void get_next_minimized_child_pos( const RECT *parent, const MINIMIZEDMETRICS *mm,
+                                          int width, int height, POINT *pos )
+{
+    BOOL next;
+
+    if (mm->iArrange & ARW_UP) /* == ARW_DOWN */
+    {
+        if (mm->iArrange & ARW_STARTTOP)
+        {
+            pos->y += height + mm->iVertGap;
+            if ((next = pos->y + height > parent->bottom))
+                pos->y = parent->top + mm->iVertGap;
+        }
+        else
+        {
+            pos->y -= height + mm->iVertGap;
+            if ((next = pos->y < parent->top))
+                pos->y = parent->bottom - mm->iVertGap - height;
+        }
+
+        if (next)
+        {
+            if (mm->iArrange & ARW_STARTRIGHT)
+                pos->x -= width + mm->iHorzGap;
+            else
+                pos->x += width + mm->iHorzGap;
+        }
+    }
+    else
+    {
+        if (mm->iArrange & ARW_STARTRIGHT)
+        {
+            pos->x -= width + mm->iHorzGap;
+            if ((next = pos->x < parent->left))
+                pos->x = parent->right - mm->iHorzGap - width;
+        }
+        else
+        {
+            pos->x += width + mm->iHorzGap;
+            if ((next = pos->x + width > parent->right))
+                pos->x = parent->left + mm->iHorzGap;
+        }
+
+        if (next)
+        {
+            if (mm->iArrange & ARW_STARTTOP)
+                pos->y += height + mm->iVertGap;
+            else
+                pos->y -= height + mm->iVertGap;
+        }
+    }
+}
+
+static POINT get_minimized_pos( HWND hwnd, POINT pt )
 {
     RECT rect, rectParent;
     HWND parent, child;
     HRGN hrgn, tmp;
-    int x, y, xspacing, yspacing;
     MINIMIZEDMETRICS metrics;
-
-    metrics.cbSize = sizeof(metrics);
-    SystemParametersInfoW( SPI_GETMINIMIZEDMETRICS, sizeof(metrics), &metrics, 0 );
+    int width, height;
 
     parent = GetAncestor( hwnd, GA_PARENT );
     if (parent == GetDesktopWindow())
@@ -868,12 +876,15 @@ static POINT WINPOS_FindIconPos( HWND hwnd, POINT pt )
     }
     else GetClientRect( parent, &rectParent );
 
-    if ((pt.x >= rectParent.left) && (pt.x + GetSystemMetrics(SM_CXICON) < rectParent.right) &&
-        (pt.y >= rectParent.top) && (pt.y + GetSystemMetrics(SM_CYICON) < rectParent.bottom))
+    if ((pt.x >= rectParent.left) && (pt.x + GetSystemMetrics( SM_CXMINIMIZED ) < rectParent.right) &&
+        (pt.y >= rectParent.top) && (pt.y + GetSystemMetrics( SM_CYMINIMIZED ) < rectParent.bottom))
         return pt;  /* The icon already has a suitable position */
 
-    xspacing = GetSystemMetrics(SM_CXICONSPACING);
-    yspacing = GetSystemMetrics(SM_CYICONSPACING);
+    width = GetSystemMetrics( SM_CXMINIMIZED );
+    height = GetSystemMetrics( SM_CYMINIMIZED );
+
+    metrics.cbSize = sizeof(metrics);
+    SystemParametersInfoW( SPI_GETMINIMIZEDMETRICS, sizeof(metrics), &metrics, 0 );
 
     /* Check if another icon already occupies this spot */
     /* FIXME: this is completely inefficient */
@@ -893,42 +904,17 @@ static POINT WINPOS_FindIconPos( HWND hwnd, POINT pt )
     }
     DeleteObject( tmp );
 
-    for (y = 0; y < (rectParent.bottom - rectParent.top) / yspacing; y++)
+    pt = get_first_minimized_child_pos( &rectParent, &metrics, width, height );
+    for (;;)
     {
-        if (metrics.iArrange & ARW_STARTTOP)
-        {
-            rect.top = rectParent.top + y * yspacing;
-            rect.bottom = rect.top + yspacing;
-        }
-        else
-        {
-            rect.bottom = rectParent.bottom - y * yspacing;
-            rect.top = rect.bottom - yspacing;
-        }
-        for (x = 0; x < (rectParent.right - rectParent.left) / xspacing; x++)
-        {
-            if (metrics.iArrange & ARW_STARTRIGHT)
-            {
-                rect.right = rectParent.right - x * xspacing;
-                rect.left = rect.right - xspacing;
-            }
-            else
-            {
-                rect.left = rectParent.left + x * xspacing;
-                rect.right = rect.left + xspacing;
-            }
-            if (!RectInRegion( hrgn, &rect ))
-            {
-                /* No window was found, so it's OK for us */
-                pt.x = rect.left + (xspacing - GetSystemMetrics(SM_CXICON)) / 2;
-                pt.y = rect.top + (yspacing - GetSystemMetrics(SM_CYICON)) / 2;
-                DeleteObject( hrgn );
-                return pt;
-            }
-        }
+        SetRect( &rect, pt.x, pt.y, pt.x + width, pt.y + height );
+        if (!RectInRegion( hrgn, &rect ))
+            break;
+
+        get_next_minimized_child_pos( &rectParent, &metrics, width, height, &pt );
     }
+
     DeleteObject( hrgn );
-    pt.x = pt.y = 0;
     return pt;
 }
 
@@ -959,11 +945,11 @@ UINT WINPOS_MinMaximize( HWND hwnd, UINT cmd, LPRECT rect )
         case SW_SHOWMINIMIZED:
         case SW_FORCEMINIMIZE:
         case SW_MINIMIZE:
-            wpl.ptMinPosition = WINPOS_FindIconPos( hwnd, wpl.ptMinPosition );
+            wpl.ptMinPosition = get_minimized_pos( hwnd, wpl.ptMinPosition );
 
             SetRect( rect, wpl.ptMinPosition.x, wpl.ptMinPosition.y,
-                     wpl.ptMinPosition.x + GetSystemMetrics(SM_CXICON),
-                     wpl.ptMinPosition.y + GetSystemMetrics(SM_CYICON) );
+                     wpl.ptMinPosition.x + GetSystemMetrics(SM_CXMINIMIZED),
+                     wpl.ptMinPosition.y + GetSystemMetrics(SM_CYMINIMIZED) );
             return SWP_NOSIZE | SWP_NOMOVE;
         }
         if (!SendMessageW( hwnd, WM_QUERYOPEN, 0, 0 )) return SWP_NOSIZE | SWP_NOMOVE;
@@ -989,12 +975,12 @@ UINT WINPOS_MinMaximize( HWND hwnd, UINT cmd, LPRECT rect )
 
         old_style = WIN_SetStyle( hwnd, WS_MINIMIZE, WS_MAXIMIZE );
 
-        wpl.ptMinPosition = WINPOS_FindIconPos( hwnd, wpl.ptMinPosition );
+        wpl.ptMinPosition = get_minimized_pos( hwnd, wpl.ptMinPosition );
 
         if (!(old_style & WS_MINIMIZE)) swpFlags |= SWP_STATECHANGED;
         SetRect( rect, wpl.ptMinPosition.x, wpl.ptMinPosition.y,
-                 wpl.ptMinPosition.x + GetSystemMetrics(SM_CXICON),
-                 wpl.ptMinPosition.y + GetSystemMetrics(SM_CYICON) );
+                 wpl.ptMinPosition.x + GetSystemMetrics(SM_CXMINIMIZED),
+                 wpl.ptMinPosition.y + GetSystemMetrics(SM_CYMINIMIZED) );
         swpFlags |= SWP_NOCOPYBITS;
         break;
 
@@ -1006,10 +992,7 @@ UINT WINPOS_MinMaximize( HWND hwnd, UINT cmd, LPRECT rect )
 
         old_style = WIN_SetStyle( hwnd, WS_MAXIMIZE, WS_MINIMIZE );
         if (old_style & WS_MINIMIZE)
-        {
             win_set_flags( hwnd, WIN_RESTORE_MAX, 0 );
-            WINPOS_ShowIconTitle( hwnd, FALSE );
-        }
 
         if (!(old_style & WS_MAXIMIZE)) swpFlags |= SWP_STATECHANGED;
         SetRect( rect, minmax.ptMaxPosition.x, minmax.ptMaxPosition.y,
@@ -1025,7 +1008,6 @@ UINT WINPOS_MinMaximize( HWND hwnd, UINT cmd, LPRECT rect )
         old_style = WIN_SetStyle( hwnd, 0, WS_MINIMIZE | WS_MAXIMIZE );
         if (old_style & WS_MINIMIZE)
         {
-            WINPOS_ShowIconTitle( hwnd, FALSE );
             if (win_get_flags( hwnd ) & WIN_RESTORE_MAX)
             {
                 /* Restore to maximized position */
@@ -1060,6 +1042,7 @@ static BOOL show_window( HWND hwnd, INT cmd )
 {
     WND *wndPtr;
     HWND parent;
+    DPI_AWARENESS_CONTEXT context;
     LONG style = GetWindowLongW( hwnd, GWL_STYLE );
     BOOL wasVisible = (style & WS_VISIBLE) != 0;
     BOOL showFlag = TRUE;
@@ -1068,10 +1051,12 @@ static BOOL show_window( HWND hwnd, INT cmd )
 
     TRACE("hwnd=%p, cmd=%d, wasVisible %d\n", hwnd, cmd, wasVisible);
 
+    context = SetThreadDpiAwarenessContext( GetWindowDpiAwarenessContext( hwnd ));
+
     switch(cmd)
     {
         case SW_HIDE:
-            if (!wasVisible) return FALSE;
+            if (!wasVisible) goto done;
             showFlag = FALSE;
             swp |= SWP_HIDEWINDOW | SWP_NOSIZE | SWP_NOMOVE;
             if (style & WS_CHILD) swp |= SWP_NOACTIVATE | SWP_NOZORDER;
@@ -1085,14 +1070,14 @@ static BOOL show_window( HWND hwnd, INT cmd )
 	case SW_SHOWMINIMIZED:
             swp |= SWP_SHOWWINDOW | SWP_FRAMECHANGED;
             swp |= WINPOS_MinMaximize( hwnd, cmd, &newPos );
-            if ((style & WS_MINIMIZE) && wasVisible) return TRUE;
+            if ((style & WS_MINIMIZE) && wasVisible) goto done;
 	    break;
 
 	case SW_SHOWMAXIMIZED: /* same as SW_MAXIMIZE */
             if (!wasVisible) swp |= SWP_SHOWWINDOW;
             swp |= SWP_FRAMECHANGED;
             swp |= WINPOS_MinMaximize( hwnd, SW_MAXIMIZE, &newPos );
-            if ((style & WS_MAXIMIZE) && wasVisible) return TRUE;
+            if ((style & WS_MAXIMIZE) && wasVisible) goto done;
             break;
 
 	case SW_SHOWNA:
@@ -1100,7 +1085,7 @@ static BOOL show_window( HWND hwnd, INT cmd )
             if (style & WS_CHILD) swp |= SWP_NOZORDER;
             break;
 	case SW_SHOW:
-            if (wasVisible) return TRUE;
+            if (wasVisible) goto done;
 	    swp |= SWP_SHOWWINDOW | SWP_NOSIZE | SWP_NOMOVE;
             if (style & WS_CHILD) swp |= SWP_NOACTIVATE | SWP_NOZORDER;
 	    break;
@@ -1120,19 +1105,19 @@ static BOOL show_window( HWND hwnd, INT cmd )
             }
             else
             {
-                if (wasVisible) return TRUE;
+                if (wasVisible) goto done;
                 swp |= SWP_NOSIZE | SWP_NOMOVE;
             }
             if (style & WS_CHILD && !(swp & SWP_STATECHANGED)) swp |= SWP_NOACTIVATE | SWP_NOZORDER;
 	    break;
         default:
-            return wasVisible;
+            goto done;
     }
 
     if ((showFlag != wasVisible || cmd == SW_SHOWNA) && cmd != SW_SHOWMAXIMIZED && !(swp & SWP_STATECHANGED))
     {
         SendMessageW( hwnd, WM_SHOWWINDOW, showFlag, 0 );
-        if (!IsWindow( hwnd )) return wasVisible;
+        if (!IsWindow( hwnd )) goto done;
     }
 
     swp = USER_Driver->pShowWindow( hwnd, cmd, &newPos, swp );
@@ -1152,8 +1137,6 @@ static BOOL show_window( HWND hwnd, INT cmd )
     {
         HWND hFocus;
 
-        WINPOS_ShowIconTitle( hwnd, FALSE );
-
         /* FIXME: This will cause the window to be activated irrespective
          * of whether it is owned by the same thread. Has to be done
          * asynchronously.
@@ -1170,12 +1153,10 @@ static BOOL show_window( HWND hwnd, INT cmd )
             if (parent == GetDesktopWindow()) parent = 0;
             SetFocus(parent);
         }
-        return wasVisible;
+        goto done;
     }
 
-    if (IsIconic(hwnd)) WINPOS_ShowIconTitle( hwnd, TRUE );
-
-    if (!(wndPtr = WIN_GetPtr( hwnd )) || wndPtr == WND_OTHER_PROCESS) return wasVisible;
+    if (!(wndPtr = WIN_GetPtr( hwnd )) || wndPtr == WND_OTHER_PROCESS) goto done;
 
     if (wndPtr->flags & WIN_NEED_SIZE)
     {
@@ -1203,6 +1184,8 @@ static BOOL show_window( HWND hwnd, INT cmd )
     /* if previous state was minimized Windows sets focus to the window */
     if (style & WS_MINIMIZE) SetFocus( hwnd );
 
+done:
+    SetThreadDpiAwarenessContext( context );
     return wasVisible;
 }
 
@@ -1336,9 +1319,9 @@ BOOL WINAPI GetWindowPlacement( HWND hwnd, WINDOWPLACEMENT *wndpl )
         wndpl->flags = WPF_RESTORETOMAXIMIZED;
     else
         wndpl->flags = 0;
-    wndpl->ptMinPosition    = pWnd->min_pos;
-    wndpl->ptMaxPosition    = pWnd->max_pos;
-    wndpl->rcNormalPosition = pWnd->normal_rect;
+    wndpl->ptMinPosition = EMPTYPOINT(pWnd->min_pos) ? pWnd->min_pos : point_win_to_thread_dpi( hwnd, pWnd->min_pos );
+    wndpl->ptMaxPosition = EMPTYPOINT(pWnd->max_pos) ? pWnd->max_pos : point_win_to_thread_dpi( hwnd, pWnd->max_pos );
+    wndpl->rcNormalPosition = rect_win_to_thread_dpi( hwnd, pWnd->normal_rect );
     WIN_ReleasePtr( pWnd );
 
     TRACE( "%p: returning min %d,%d max %d,%d normal %s\n",
@@ -1413,9 +1396,9 @@ static BOOL WINPOS_SetPlacement( HWND hwnd, const WINDOWPLACEMENT *wndpl, UINT f
 
     if (!pWnd || pWnd == WND_OTHER_PROCESS || pWnd == WND_DESKTOP) return FALSE;
 
-    if( flags & PLACE_MIN ) pWnd->min_pos = wp.ptMinPosition;
-    if( flags & PLACE_MAX ) pWnd->max_pos = wp.ptMaxPosition;
-    if( flags & PLACE_RECT) pWnd->normal_rect = wp.rcNormalPosition;
+    if (flags & PLACE_MIN) pWnd->min_pos = point_thread_to_win_dpi( hwnd, wp.ptMinPosition );
+    if (flags & PLACE_MAX) pWnd->max_pos = point_thread_to_win_dpi( hwnd, wp.ptMaxPosition );
+    if (flags & PLACE_RECT) pWnd->normal_rect = rect_thread_to_win_dpi( hwnd, wp.rcNormalPosition );
 
     style = pWnd->dwStyle;
 
@@ -1425,7 +1408,6 @@ static BOOL WINPOS_SetPlacement( HWND hwnd, const WINDOWPLACEMENT *wndpl, UINT f
     {
         if (flags & PLACE_MIN)
         {
-            WINPOS_ShowIconTitle( hwnd, FALSE );
             SetWindowPos( hwnd, 0, wp.ptMinPosition.x, wp.ptMinPosition.y, 0, 0,
                           SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE );
         }
@@ -1446,8 +1428,6 @@ static BOOL WINPOS_SetPlacement( HWND hwnd, const WINDOWPLACEMENT *wndpl, UINT f
 
     if (IsIconic( hwnd ))
     {
-        if (GetWindowLongW( hwnd, GWL_STYLE ) & WS_VISIBLE) WINPOS_ShowIconTitle( hwnd, TRUE );
-
         /* SDK: ...valid only the next time... */
         if( wndpl->flags & WPF_RESTORETOMAXIMIZED )
             win_set_flags( hwnd, WIN_RESTORE_MAX, 0 );
@@ -1643,6 +1623,23 @@ static void dump_winpos_flags(UINT flags)
     TRACE("\n");
 }
 
+
+/***********************************************************************
+ *           map_dpi_winpos
+ */
+static void map_dpi_winpos( WINDOWPOS *winpos )
+{
+    UINT dpi_from = get_thread_dpi();
+    UINT dpi_to = GetDpiForWindow( winpos->hwnd );
+
+    if (!dpi_from) dpi_from = get_win_monitor_dpi( winpos->hwnd );
+    if (dpi_from == dpi_to) return;
+    winpos->x  = MulDiv( winpos->x, dpi_to, dpi_from );
+    winpos->y  = MulDiv( winpos->y, dpi_to, dpi_from );
+    winpos->cx = MulDiv( winpos->cx, dpi_to, dpi_from );
+    winpos->cy = MulDiv( winpos->cy, dpi_to, dpi_from );
+}
+
 /***********************************************************************
  *           SWP_DoWinPosChanging
  */
@@ -1664,14 +1661,14 @@ static BOOL SWP_DoWinPosChanging( WINDOWPOS *pWinpos, RECT *old_window_rect, REC
 
     WIN_GetRectangles( pWinpos->hwnd, COORDS_PARENT, old_window_rect, old_client_rect );
     *new_window_rect = *old_window_rect;
-    *new_client_rect = (wndPtr->dwStyle & WS_MINIMIZE) ? *old_window_rect : *old_client_rect;
+    *new_client_rect = *old_client_rect;
 
     if (!(pWinpos->flags & SWP_NOSIZE))
     {
         if (wndPtr->dwStyle & WS_MINIMIZE)
         {
-            new_window_rect->right  = new_window_rect->left + GetSystemMetrics(SM_CXICON);
-            new_window_rect->bottom = new_window_rect->top + GetSystemMetrics(SM_CYICON);
+            new_window_rect->right  = new_window_rect->left + GetSystemMetrics(SM_CXMINIMIZED);
+            new_window_rect->bottom = new_window_rect->top + GetSystemMetrics(SM_CYMINIMIZED);
         }
         else
         {
@@ -1691,10 +1688,9 @@ static BOOL SWP_DoWinPosChanging( WINDOWPOS *pWinpos, RECT *old_window_rect, REC
     }
     pWinpos->flags |= SWP_NOCLIENTMOVE | SWP_NOCLIENTSIZE;
 
-    TRACE( "hwnd %p, after %p, swp %d,%d %dx%d flags %08x\n",
+    TRACE( "hwnd %p, after %p, swp %d,%d %dx%d flags %08x current %s style %08x new %s\n",
            pWinpos->hwnd, pWinpos->hwndInsertAfter, pWinpos->x, pWinpos->y,
-           pWinpos->cx, pWinpos->cy, pWinpos->flags );
-    TRACE( "current %s style %08x new %s\n",
+           pWinpos->cx, pWinpos->cy, pWinpos->flags,
            wine_dbgstr_rect( old_window_rect ), wndPtr->dwStyle,
            wine_dbgstr_rect( new_window_rect ));
 
@@ -2216,7 +2212,9 @@ BOOL USER_SetWindowPos( WINDOWPOS * winpos, int parent_x, int parent_y )
 {
     RECT old_window_rect, old_client_rect, new_window_rect, new_client_rect, valid_rects[2];
     UINT orig_flags;
-    
+    BOOL ret = FALSE;
+    DPI_AWARENESS_CONTEXT context;
+
     orig_flags = winpos->flags;
 
     /* First, check z-order arguments.  */
@@ -2256,11 +2254,13 @@ BOOL USER_SetWindowPos( WINDOWPOS * winpos, int parent_x, int parent_y )
         else if (winpos->cy > 32767) winpos->cy = 32767;
     }
 
+    context = SetThreadDpiAwarenessContext( GetWindowDpiAwarenessContext( winpos->hwnd ));
+
     if (!SWP_DoWinPosChanging( winpos, &old_window_rect, &old_client_rect,
-                               &new_window_rect, &new_client_rect )) return FALSE;
+                               &new_window_rect, &new_client_rect )) goto done;
 
     /* Fix redundant flags */
-    if (!fixup_flags( winpos, &old_window_rect, parent_x, parent_y )) return FALSE;
+    if (!fixup_flags( winpos, &old_window_rect, parent_x, parent_y )) goto done;
 
     if((winpos->flags & (SWP_NOZORDER | SWP_HIDEWINDOW | SWP_SHOWWINDOW)) != SWP_NOZORDER)
     {
@@ -2275,8 +2275,7 @@ BOOL USER_SetWindowPos( WINDOWPOS * winpos, int parent_x, int parent_y )
 
     if (!set_window_pos( winpos->hwnd, winpos->hwndInsertAfter, winpos->flags,
                          &new_window_rect, &new_client_rect, valid_rects ))
-        return FALSE;
-
+        goto done;
 
 
     if( winpos->flags & SWP_HIDEWINDOW )
@@ -2329,7 +2328,10 @@ BOOL USER_SetWindowPos( WINDOWPOS * winpos, int parent_x, int parent_y )
         winpos->cy = new_window_rect.bottom - new_window_rect.top;
         SendMessageW( winpos->hwnd, WM_WINDOWPOSCHANGED, 0, (LPARAM)winpos );
     }
-    return TRUE;
+    ret = TRUE;
+done:
+    SetThreadDpiAwarenessContext( context );
+    return ret;
 }
 
 /***********************************************************************
@@ -2357,7 +2359,9 @@ BOOL WINAPI SetWindowPos( HWND hwnd, HWND hwndInsertAfter,
     winpos.cx = cx;
     winpos.cy = cy;
     winpos.flags = flags;
-    
+
+    map_dpi_winpos( &winpos );
+
     if (WIN_IsCurrentThread( hwnd ))
         return USER_SetWindowPos( &winpos, 0, 0 );
 
@@ -2411,17 +2415,26 @@ HDWP WINAPI DeferWindowPos( HDWP hdwp, HWND hwnd, HWND hwndAfter,
     DWP *pDWP;
     int i;
     HDWP retvalue = hdwp;
+    WINDOWPOS winpos;
 
     TRACE("hdwp %p, hwnd %p, after %p, %d,%d (%dx%d), flags %08x\n",
           hdwp, hwnd, hwndAfter, x, y, cx, cy, flags);
 
-    hwnd = WIN_GetFullHandle( hwnd );
-    if (is_desktop_window( hwnd ) || !IsWindow( hwnd ))
+    winpos.hwnd = WIN_GetFullHandle( hwnd );
+    if (is_desktop_window( winpos.hwnd ) || !IsWindow( winpos.hwnd ))
     {
         SetLastError( ERROR_INVALID_WINDOW_HANDLE );
         return 0;
     }
 
+    winpos.hwndInsertAfter = WIN_GetFullHandle(hwndAfter);
+    winpos.flags = flags;
+    winpos.x = x;
+    winpos.y = y;
+    winpos.cx = cx;
+    winpos.cy = cy;
+    map_dpi_winpos( &winpos );
+
     if (!(pDWP = get_user_handle_ptr( hdwp, USER_DWP ))) return 0;
     if (pDWP == OBJ_OTHER_PROCESS)
     {
@@ -2431,22 +2444,22 @@ HDWP WINAPI DeferWindowPos( HDWP hdwp, HWND hwnd, HWND hwndAfter,
 
     for (i = 0; i < pDWP->actualCount; i++)
     {
-        if (pDWP->winPos[i].hwnd == hwnd)
+        if (pDWP->winPos[i].hwnd == winpos.hwnd)
         {
               /* Merge with the other changes */
             if (!(flags & SWP_NOZORDER))
             {
-                pDWP->winPos[i].hwndInsertAfter = WIN_GetFullHandle(hwndAfter);
+                pDWP->winPos[i].hwndInsertAfter = winpos.hwndInsertAfter;
             }
             if (!(flags & SWP_NOMOVE))
             {
-                pDWP->winPos[i].x = x;
-                pDWP->winPos[i].y = y;
+                pDWP->winPos[i].x = winpos.x;
+                pDWP->winPos[i].y = winpos.y;
             }
             if (!(flags & SWP_NOSIZE))
             {
-                pDWP->winPos[i].cx = cx;
-                pDWP->winPos[i].cy = cy;
+                pDWP->winPos[i].cx = winpos.cx;
+                pDWP->winPos[i].cy = winpos.cy;
             }
             pDWP->winPos[i].flags &= flags | ~(SWP_NOSIZE | SWP_NOMOVE |
                                                SWP_NOZORDER | SWP_NOREDRAW |
@@ -2469,14 +2482,7 @@ HDWP WINAPI DeferWindowPos( HDWP hdwp, HWND hwnd, HWND hwndAfter,
         pDWP->suggestedCount *= 2;
         pDWP->winPos = newpos;
     }
-    pDWP->winPos[pDWP->actualCount].hwnd = hwnd;
-    pDWP->winPos[pDWP->actualCount].hwndInsertAfter = hwndAfter;
-    pDWP->winPos[pDWP->actualCount].x = x;
-    pDWP->winPos[pDWP->actualCount].y = y;
-    pDWP->winPos[pDWP->actualCount].cx = cx;
-    pDWP->winPos[pDWP->actualCount].cy = cy;
-    pDWP->winPos[pDWP->actualCount].flags = flags;
-    pDWP->actualCount++;
+    pDWP->winPos[pDWP->actualCount++] = winpos;
 END:
     release_user_handle_ptr( pDWP );
     return retvalue;
@@ -2523,14 +2529,16 @@ BOOL WINAPI EndDeferWindowPos( HDWP hdwp )
  */
 UINT WINAPI ArrangeIconicWindows( HWND parent )
 {
+    int width, height, count = 0;
     RECT rectParent;
     HWND hwndChild;
-    INT x, y, xspacing, yspacing;
     POINT pt;
     MINIMIZEDMETRICS metrics;
 
     metrics.cbSize = sizeof(metrics);
     SystemParametersInfoW( SPI_GETMINIMIZEDMETRICS, sizeof(metrics), &metrics, 0 );
+    width = GetSystemMetrics( SM_CXMINIMIZED );
+    height = GetSystemMetrics( SM_CYMINIMIZED );
 
     if (parent == GetDesktopWindow())
     {
@@ -2543,41 +2551,21 @@ UINT WINAPI ArrangeIconicWindows( HWND parent )
     }
     else GetClientRect( parent, &rectParent );
 
-    x = y = 0;
-    xspacing = GetSystemMetrics(SM_CXICONSPACING);
-    yspacing = GetSystemMetrics(SM_CYICONSPACING);
+    pt = get_first_minimized_child_pos( &rectParent, &metrics, width, height );
 
     hwndChild = GetWindow( parent, GW_CHILD );
     while (hwndChild)
     {
         if( IsIconic( hwndChild ) )
         {
-            WINPOS_ShowIconTitle( hwndChild, FALSE );
-
-            if (metrics.iArrange & ARW_STARTRIGHT)
-                pt.x = rectParent.right - (x + 1) * xspacing;
-            else
-                pt.x = rectParent.left + x * xspacing;
-            if (metrics.iArrange & ARW_STARTTOP)
-                pt.y = rectParent.top + y * yspacing;
-            else
-                pt.y = rectParent.bottom - (y + 1) * yspacing;
-
-            SetWindowPos( hwndChild, 0, pt.x + (xspacing - GetSystemMetrics(SM_CXICON)) / 2,
-                          pt.y + (yspacing - GetSystemMetrics(SM_CYICON)) / 2, 0, 0,
+            SetWindowPos( hwndChild, 0, pt.x, pt.y, 0, 0,
                           SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE );
-	    if( IsWindow(hwndChild) )
-                WINPOS_ShowIconTitle(hwndChild , TRUE );
-
-            if (++x >= (rectParent.right - rectParent.left) / xspacing)
-            {
-                x = 0;
-                y++;
-            }
+            get_next_minimized_child_pos( &rectParent, &metrics, width, height, &pt );
+            count++;
         }
         hwndChild = GetWindow( hwndChild, GW_HWNDNEXT );
     }
-    return yspacing;
+    return count;
 }
 
 
@@ -2721,12 +2709,10 @@ void WINPOS_SysCommandSizeMove( HWND hwnd, WPARAM wParam )
     HWND parent;
     LONG hittest = (LONG)(wParam & 0x0f);
     WPARAM syscommand = wParam & 0xfff0;
-    HCURSOR hDragCursor = 0, hOldCursor = 0;
     MINMAXINFO minmax;
     POINT capturePoint, pt;
     LONG style = GetWindowLongW( hwnd, GWL_STYLE );
     BOOL    thickframe = HAS_THICKFRAME( style );
-    BOOL    iconic = style & WS_MINIMIZE;
     BOOL    moved = FALSE;
     DWORD     dwPoint = GetMessagePos ();
     BOOL DragFullWindows = TRUE;
@@ -2806,13 +2792,6 @@ void WINPOS_SysCommandSizeMove( HWND hwnd, WPARAM wParam )
     /* Retrieve a default cache DC (without using the window style) */
     hdc = GetDCEx( parent, 0, DCX_CACHE );
 
-    if( iconic ) /* create a cursor for dragging */
-    {
-        hDragCursor = (HCURSOR)GetClassLongPtrW( hwnd, GCLP_HICON);
-        if( !hDragCursor ) hDragCursor = (HCURSOR)SendMessageW( hwnd, WM_QUERYDRAGICON, 0, 0L);
-        if( !hDragCursor ) iconic = FALSE;
-    }
-
     /* we only allow disabling the full window drag for child windows */
     if (parent) SystemParametersInfoW( SPI_GETDRAGFULLWINDOWS, 0, &DragFullWindows, 0 );
 
@@ -2882,21 +2861,14 @@ void WINPOS_SysCommandSizeMove( HWND hwnd, WPARAM wParam )
             if( !moved )
             {
                 moved = TRUE;
-
-                if( iconic ) /* ok, no system popup tracking */
-                {
-                    hOldCursor = SetCursor(hDragCursor);
-                    ShowCursor( TRUE );
-                    WINPOS_ShowIconTitle( hwnd, FALSE );
-                }
-                else if(!DragFullWindows)
+                if (!DragFullWindows)
                     draw_moving_frame( parent, hdc, &sizingRect, thickframe );
             }
 
             if (msg.message == WM_KEYDOWN) SetCursorPos( pt.x, pt.y );
             else
             {
-                if(!iconic && !DragFullWindows) draw_moving_frame( parent, hdc, &sizingRect, thickframe );
+                if (!DragFullWindows) draw_moving_frame( parent, hdc, &sizingRect, thickframe );
                 if (hittest == HTCAPTION) OffsetRect( &sizingRect, dx, dy );
                 if (ON_LEFT_BORDER(hittest)) sizingRect.left += dx;
                 else if (ON_RIGHT_BORDER(hittest)) sizingRect.right += dx;
@@ -2916,32 +2888,21 @@ void WINPOS_SysCommandSizeMove( HWND hwnd, WPARAM wParam )
                 else
                     SendMessageW( hwnd, WM_MOVING, 0, (LPARAM)&sizingRect );
 
-                if (!iconic)
+                if (!DragFullWindows)
+                    draw_moving_frame( parent, hdc, &sizingRect, thickframe );
+                else
                 {
-                    if(!DragFullWindows)
-                        draw_moving_frame( parent, hdc, &sizingRect, thickframe );
-                    else
-                    {
-                        RECT rect = sizingRect;
-                        MapWindowPoints( 0, parent, (POINT *)&rect, 2 );
-                        SetWindowPos( hwnd, 0, rect.left, rect.top,
-                                      rect.right - rect.left, rect.bottom - rect.top,
-                                      ( hittest == HTCAPTION ) ? SWP_NOSIZE : 0 );
-                    }
+                    RECT rect = sizingRect;
+                    MapWindowPoints( 0, parent, (POINT *)&rect, 2 );
+                    SetWindowPos( hwnd, 0, rect.left, rect.top,
+                                  rect.right - rect.left, rect.bottom - rect.top,
+                                  (hittest == HTCAPTION) ? SWP_NOSIZE : 0 );
                 }
             }
         }
     }
 
-    if( iconic )
-    {
-        if( moved ) /* restore cursors, show icon title later on */
-        {
-            ShowCursor( FALSE );
-            SetCursor( hOldCursor );
-        }
-    }
-    else if (moved && !DragFullWindows)
+    if (moved && !DragFullWindows)
     {
         draw_moving_frame( parent, hdc, &sizingRect, thickframe );
     }
@@ -2965,7 +2926,7 @@ void WINPOS_SysCommandSizeMove( HWND hwnd, WPARAM wParam )
         if (!((msg.message == WM_KEYDOWN) && (msg.wParam == VK_ESCAPE)) )
         {
             /* NOTE: SWP_NOACTIVATE prevents document window activation in Word 6 */
-            if(!DragFullWindows || iconic)
+            if (!DragFullWindows)
                 SetWindowPos( hwnd, 0, sizingRect.left, sizingRect.top,
                               sizingRect.right - sizingRect.left,
                               sizingRect.bottom - sizingRect.top,
@@ -2991,6 +2952,5 @@ void WINPOS_SysCommandSizeMove( HWND hwnd, WPARAM wParam )
                 SendMessageW( hwnd, WM_SYSCOMMAND,
                               SC_MOUSEMENU + HTSYSMENU, MAKELONG(pt.x,pt.y));
         }
-        else WINPOS_ShowIconTitle( hwnd, TRUE );
     }
 }
