@@ -114,9 +114,14 @@ ULONG CDECL wined3d_swapchain_decref(struct wined3d_swapchain *swapchain)
 
     if (!refcount)
     {
+        struct wined3d_device *device;
+
         wined3d_mutex_lock();
 
-        wined3d_cs_finish(swapchain->device->cs, WINED3D_CS_QUEUE_DEFAULT);
+        device = swapchain->device;
+        if (device->swapchain_count && device->swapchains[0] == swapchain)
+            wined3d_device_uninit_3d(device);
+        wined3d_cs_finish(device->cs, WINED3D_CS_QUEUE_DEFAULT);
 
         swapchain_cleanup(swapchain);
         swapchain->parent_ops->wined3d_object_destroyed(swapchain->parent);
@@ -157,9 +162,9 @@ HRESULT CDECL wined3d_swapchain_present(struct wined3d_swapchain *swapchain,
 {
     RECT s, d;
 
-    TRACE("swapchain %p, src_rect %s, dst_rect %s, dst_window_override %p, flags %#x.\n",
+    TRACE("swapchain %p, src_rect %s, dst_rect %s, dst_window_override %p, swap_interval %u, flags %#x.\n",
             swapchain, wine_dbgstr_rect(src_rect), wine_dbgstr_rect(dst_rect),
-            dst_window_override, flags);
+            dst_window_override, swap_interval, flags);
 
     if (flags)
         FIXME("Ignoring flags %#x.\n", flags);
@@ -348,6 +353,27 @@ static void swapchain_blit(const struct wined3d_swapchain *swapchain,
     wined3d_texture_invalidate_location(texture, 0, WINED3D_LOCATION_DRAWABLE);
 }
 
+static void swapchain_gl_set_swap_interval(struct wined3d_swapchain *swapchain,
+        struct wined3d_context *context, unsigned int swap_interval)
+{
+    const struct wined3d_gl_info *gl_info = context->gl_info;
+
+    swap_interval = swap_interval <= 4 ? swap_interval : 1;
+    if (swapchain->swap_interval == swap_interval)
+        return;
+
+    swapchain->swap_interval = swap_interval;
+
+    if (!gl_info->supported[WGL_EXT_SWAP_CONTROL])
+        return;
+
+    if (!GL_EXTCALL(wglSwapIntervalEXT(swap_interval)))
+    {
+        ERR("Failed to set swap interval %u for context %p, last error %#x.\n",
+                swap_interval, context, GetLastError());
+    }
+}
+
 /* Context activation is done by the caller. */
 static void wined3d_swapchain_gl_rotate(struct wined3d_swapchain *swapchain, struct wined3d_context *context)
 {
@@ -396,7 +422,7 @@ static void wined3d_swapchain_gl_rotate(struct wined3d_swapchain *swapchain, str
 }
 
 static void swapchain_gl_present(struct wined3d_swapchain *swapchain,
-        const RECT *src_rect, const RECT *dst_rect, DWORD flags)
+        const RECT *src_rect, const RECT *dst_rect, unsigned int swap_interval, DWORD flags)
 {
     struct wined3d_texture *back_buffer = swapchain->back_buffers[0];
     const struct wined3d_fb_state *fb = &swapchain->device->cs->fb;
@@ -416,6 +442,8 @@ static void swapchain_gl_present(struct wined3d_swapchain *swapchain,
 
     gl_info = context->gl_info;
 
+    swapchain_gl_set_swap_interval(swapchain, context, swap_interval);
+
     if ((logo_texture = swapchain->device->logo_texture))
     {
         RECT rect = {0, 0, logo_texture->resource.width, logo_texture->resource.height};
@@ -597,7 +625,7 @@ static void swapchain_gdi_frontbuffer_updated(struct wined3d_swapchain *swapchai
 }
 
 static void swapchain_gdi_present(struct wined3d_swapchain *swapchain,
-        const RECT *src_rect, const RECT *dst_rect, DWORD flags)
+        const RECT *src_rect, const RECT *dst_rect, unsigned int swap_interval, DWORD flags)
 {
     struct wined3d_dc_info *front, *back;
     HBITMAP bitmap;
@@ -689,59 +717,11 @@ static void wined3d_swapchain_apply_sample_count_override(const struct wined3d_s
     *quality = 0;
 }
 
-void wined3d_swapchain_set_swap_interval(struct wined3d_swapchain *swapchain,
-        unsigned int swap_interval)
-{
-    const struct wined3d_gl_info *gl_info;
-    struct wined3d_context *context;
-
-    swap_interval = swap_interval <= 4 ? swap_interval : 1;
-    if (swapchain->swap_interval == swap_interval)
-        return;
-
-    swapchain->swap_interval = swap_interval;
-
-    context = context_acquire(swapchain->device, swapchain->front_buffer, 0);
-    gl_info = context->gl_info;
-
-    if (gl_info->supported[WGL_EXT_SWAP_CONTROL])
-    {
-        if (!GL_EXTCALL(wglSwapIntervalEXT(swap_interval)))
-            ERR("wglSwapIntervalEXT failed to set swap interval %d for context %p, last error %#x.\n",
-                    swap_interval, context, GetLastError());
-    }
-
-    context_release(context);
-}
-
 static void wined3d_swapchain_cs_init(void *object)
 {
     struct wined3d_swapchain *swapchain = object;
-    const struct wined3d_adapter *adapter;
-    unsigned int i;
 
-    static const enum wined3d_format_id formats[] =
-    {
-        WINED3DFMT_D24_UNORM_S8_UINT,
-        WINED3DFMT_D32_UNORM,
-        WINED3DFMT_R24_UNORM_X8_TYPELESS,
-        WINED3DFMT_D16_UNORM,
-        WINED3DFMT_S1_UINT_D15_UNORM,
-    };
-
-    adapter = swapchain->device->adapter;
-
-    /* Without ORM_FBO, switching the depth/stencil format is hard. Always
-     * request a depth/stencil buffer in the likely case it's needed later. */
-    for (i = 0; i < ARRAY_SIZE(formats); ++i)
-    {
-        swapchain->ds_format = wined3d_get_format(adapter, formats[i], WINED3D_BIND_DEPTH_STENCIL);
-        if ((swapchain->context[0] = context_create(swapchain, swapchain->front_buffer, swapchain->ds_format)))
-            break;
-        TRACE("Depth stencil format %s is not supported, trying next format.\n", debug_d3dformat(formats[i]));
-    }
-
-    if (!swapchain->context[0])
+    if (!(swapchain->context[0] = context_create(swapchain)))
     {
         WARN("Failed to create context.\n");
         return;
@@ -1055,6 +1035,20 @@ HRESULT CDECL wined3d_swapchain_create(struct wined3d_device *device, struct win
         return hr;
     }
 
+    if (desc->flags & WINED3D_SWAPCHAIN_IMPLICIT)
+    {
+        wined3d_mutex_lock();
+        if (FAILED(hr = wined3d_device_set_implicit_swapchain(device, object)))
+        {
+            wined3d_cs_finish(device->cs, WINED3D_CS_QUEUE_DEFAULT);
+            swapchain_cleanup(object);
+            wined3d_mutex_unlock();
+            heap_free(swapchain);
+            return hr;
+        }
+        wined3d_mutex_unlock();
+    }
+
     TRACE("Created swapchain %p.\n", object);
     *swapchain = object;
 
@@ -1068,7 +1062,7 @@ static struct wined3d_context *swapchain_create_context(struct wined3d_swapchain
 
     TRACE("Creating a new context for swapchain %p, thread %u.\n", swapchain, GetCurrentThreadId());
 
-    if (!(ctx = context_create(swapchain, swapchain->front_buffer, swapchain->ds_format)))
+    if (!(ctx = context_create(swapchain)))
     {
         ERR("Failed to create a new context for the swapchain\n");
         return NULL;
@@ -1078,7 +1072,7 @@ static struct wined3d_context *swapchain_create_context(struct wined3d_swapchain
     if (!(ctx_array = heap_calloc(swapchain->num_contexts + 1, sizeof(*ctx_array))))
     {
         ERR("Out of memory when trying to allocate a new context array\n");
-        context_destroy(swapchain->device, ctx);
+        wined3d_context_destroy(ctx);
         return NULL;
     }
     memcpy(ctx_array, swapchain->context, sizeof(*ctx_array) * swapchain->num_contexts);
@@ -1097,7 +1091,7 @@ void swapchain_destroy_contexts(struct wined3d_swapchain *swapchain)
 
     for (i = 0; i < swapchain->num_contexts; ++i)
     {
-        context_destroy(swapchain->device, swapchain->context[i]);
+        wined3d_context_destroy(swapchain->context[i]);
     }
     heap_free(swapchain->context);
     swapchain->num_contexts = 0;
