@@ -19,33 +19,23 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-#include "wine/port.h"
-
 #include <stdarg.h>
-#if defined(__MINGW32__) || defined (_MSC_VER)
-#include <winsock2.h>
-#elif defined(HAVE_UNISTD_H)
-#include <unistd.h>
-#endif
-
+#include <stdio.h>
 #include <windef.h>
 #include <winbase.h>
 #include <wincon.h>
 #include <winnls.h>
 #include <winuser.h>
 
-#include <wine/unicode.h>
-
 #include "hostname.h"
 
-static int hostname_vprintfW(const WCHAR *msg, va_list va_args)
+static int hostname_vprintfW(const WCHAR *msg, __ms_va_list va_args)
 {
     int wlen;
     DWORD count, ret;
     WCHAR msg_buffer[8192];
 
-    wlen = vsprintfW(msg_buffer, msg, va_args);
+    wlen = vswprintf(msg_buffer, ARRAY_SIZE(msg_buffer), msg, va_args);
 
     ret = WriteConsoleW(GetStdHandle(STD_OUTPUT_HANDLE), msg_buffer, wlen, &count, NULL);
     if (!ret)
@@ -72,29 +62,29 @@ static int hostname_vprintfW(const WCHAR *msg, va_list va_args)
     return count;
 }
 
-static int hostname_printfW(const WCHAR *msg, ...)
+static int WINAPIV hostname_printfW(const WCHAR *msg, ...)
 {
-    va_list va_args;
+    __ms_va_list va_args;
     int len;
 
-    va_start(va_args, msg);
+    __ms_va_start(va_args, msg);
     len = hostname_vprintfW(msg, va_args);
-    va_end(va_args);
+    __ms_va_end(va_args);
 
     return len;
 }
 
-static int hostname_message_printfW(int msg, ...)
+static int WINAPIV hostname_message_printfW(int msg, ...)
 {
-    va_list va_args;
+    __ms_va_list va_args;
     WCHAR msg_buffer[8192];
     int len;
 
     LoadStringW(GetModuleHandleW(NULL), msg, msg_buffer, ARRAY_SIZE(msg_buffer));
 
-    va_start(va_args, msg);
+    __ms_va_start(va_args, msg);
     len = hostname_vprintfW(msg_buffer, va_args);
-    va_end(va_args);
+    __ms_va_end(va_args);
 
     return len;
 }
@@ -109,17 +99,23 @@ static int hostname_message(int msg)
     return hostname_printfW(formatW, msg_buffer);
 }
 
-static void display_computer_name(void)
+static int display_computer_name(void)
 {
     static const WCHAR fmtW[] = {'%','s','\r','\n',0};
 
-    char nameA[256];
-    WCHAR nameW[256];
+    WCHAR name[MAX_COMPUTERNAME_LENGTH + 1];
+    DWORD size = ARRAY_SIZE(name);
+    BOOL ret;
 
-    gethostname(nameA, sizeof(nameA));
-    MultiByteToWideChar(CP_UNIXCP, 0, nameA, sizeof(nameA), nameW, ARRAY_SIZE(nameW));
+    ret = GetComputerNameW(name, &size);
+    if (!ret)
+    {
+        hostname_message_printfW(STRING_CANNOT_GET_HOSTNAME, GetLastError());
+        return 1;
+    }
 
-    hostname_printfW(fmtW, nameW);
+    hostname_printfW(fmtW, name);
+    return 0;
 }
 
 int wmain(int argc, WCHAR *argv[])
@@ -130,7 +126,7 @@ int wmain(int argc, WCHAR *argv[])
 
         unsigned int i;
 
-        if (!strncmpW(argv[1], slashHelpW, ARRAY_SIZE(slashHelpW) - 1))
+        if (!wcsncmp(argv[1], slashHelpW, ARRAY_SIZE(slashHelpW) - 1))
         {
             hostname_message(STRING_USAGE);
             return 1;
@@ -163,7 +159,5 @@ int wmain(int argc, WCHAR *argv[])
         }
     }
 
-    display_computer_name();
-
-    return 0;
+    return display_computer_name();
 }
