@@ -20,6 +20,7 @@
 
 #include "config.h"
 #include "wine/port.h"
+#include "wine/rbtree.h"
 
 #include <assert.h>
 #include <fcntl.h>
@@ -75,6 +76,7 @@ static const struct object_ops irp_call_ops =
     no_link_name,                     /* link_name */
     NULL,                             /* unlink_name */
     no_open_file,                     /* open_file */
+    no_kernel_obj_list,               /* get_kernel_obj_list */
     no_close_handle,                  /* close_handle */
     irp_call_destroy                  /* destroy */
 };
@@ -84,9 +86,11 @@ static const struct object_ops irp_call_ops =
 
 struct device_manager
 {
-    struct object          obj;           /* object header */
-    struct list            devices;       /* list of devices */
-    struct list            requests;      /* list of pending irps across all devices */
+    struct object          obj;            /* object header */
+    struct list            devices;        /* list of devices */
+    struct list            requests;       /* list of pending irps across all devices */
+    struct irp_call       *current_call;   /* call currently executed on client side */
+    struct wine_rb_tree    kernel_objects; /* map of objects that have client side pointer associated */
 };
 
 static void device_manager_dump( struct object *obj, int verbose );
@@ -111,6 +115,7 @@ static const struct object_ops device_manager_ops =
     no_link_name,                     /* link_name */
     NULL,                             /* unlink_name */
     no_open_file,                     /* open_file */
+    no_kernel_obj_list,               /* get_kernel_obj_list */
     no_close_handle,                  /* close_handle */
     device_manager_destroy            /* destroy */
 };
@@ -123,7 +128,7 @@ struct device
     struct object          obj;           /* object header */
     struct device_manager *manager;       /* manager for this device (or NULL if deleted) */
     char                  *unix_path;     /* path to unix device if any */
-    client_ptr_t           user_ptr;      /* opaque ptr for client side */
+    struct list            kernel_object; /* list of kernel object pointers */
     struct list            entry;         /* entry in device manager list */
     struct list            files;         /* list of open files */
 };
@@ -133,6 +138,7 @@ static struct object_type *device_get_type( struct object *obj );
 static void device_destroy( struct object *obj );
 static struct object *device_open_file( struct object *obj, unsigned int access,
                                         unsigned int sharing, unsigned int options );
+static struct list *device_get_kernel_obj_list( struct object *obj );
 
 static const struct object_ops device_ops =
 {
@@ -152,6 +158,7 @@ static const struct object_ops device_ops =
     directory_link_name,              /* link_name */
     default_unlink_name,              /* unlink_name */
     device_open_file,                 /* open_file */
+    device_get_kernel_obj_list,       /* get_kernel_obj_list */
     no_close_handle,                  /* close_handle */
     device_destroy                    /* destroy */
 };
@@ -164,13 +171,15 @@ struct device_file
     struct object          obj;           /* object header */
     struct device         *device;        /* device for this file */
     struct fd             *fd;            /* file descriptor for irp */
-    client_ptr_t           user_ptr;      /* opaque ptr for client side */
+    struct list            kernel_object; /* list of kernel object pointers */
+    int                    closed;        /* closed file flag */
     struct list            entry;         /* entry in device list */
     struct list            requests;      /* list of pending irp requests */
 };
 
 static void device_file_dump( struct object *obj, int verbose );
 static struct fd *device_file_get_fd( struct object *obj );
+static struct list *device_file_get_kernel_obj_list( struct object *obj );
 static int device_file_close_handle( struct object *obj, struct process *process, obj_handle_t handle );
 static void device_file_destroy( struct object *obj );
 static enum server_fd_type device_file_get_fd_type( struct fd *fd );
@@ -197,6 +206,7 @@ static const struct object_ops device_file_ops =
     no_link_name,                     /* link_name */
     NULL,                             /* unlink_name */
     no_open_file,                     /* open_file */
+    device_file_get_kernel_obj_list,  /* get_kernel_obj_list */
     device_file_close_handle,         /* close_handle */
     device_file_destroy               /* destroy */
 };
@@ -217,6 +227,86 @@ static const struct fd_ops device_file_fd_ops =
 };
 
 
+struct list *no_kernel_obj_list( struct object *obj )
+{
+    return NULL;
+}
+
+struct kernel_object
+{
+    struct device_manager  *manager;
+    client_ptr_t            user_ptr;
+    struct object          *object;
+    int                     owned;
+    struct list             list_entry;
+    struct wine_rb_entry    rb_entry;
+};
+
+static int compare_kernel_object( const void *k, const struct wine_rb_entry *entry )
+{
+    struct kernel_object *ptr = WINE_RB_ENTRY_VALUE( entry, struct kernel_object, rb_entry );
+    return memcmp( k, &ptr->user_ptr, sizeof(client_ptr_t) );
+}
+
+static struct kernel_object *kernel_object_from_obj( struct device_manager *manager, struct object *obj )
+{
+    struct kernel_object *kernel_object;
+    struct list *list;
+
+    if (!(list = obj->ops->get_kernel_obj_list( obj ))) return NULL;
+    LIST_FOR_EACH_ENTRY( kernel_object, list, struct kernel_object, list_entry )
+    {
+        if (kernel_object->manager != manager) continue;
+        return kernel_object;
+    }
+    return NULL;
+}
+
+static client_ptr_t get_kernel_object_ptr( struct device_manager *manager, struct object *obj )
+{
+    struct kernel_object *kernel_object = kernel_object_from_obj( manager, obj );
+    return kernel_object ? kernel_object->user_ptr : 0;
+}
+
+static struct kernel_object *set_kernel_object( struct device_manager *manager, struct object *obj, client_ptr_t user_ptr )
+{
+    struct kernel_object *kernel_object;
+    struct list *list;
+
+    if (!(list = obj->ops->get_kernel_obj_list( obj ))) return NULL;
+
+    if (!(kernel_object = malloc( sizeof(*kernel_object) ))) return NULL;
+    kernel_object->manager  = manager;
+    kernel_object->user_ptr = user_ptr;
+    kernel_object->object   = obj;
+    kernel_object->owned    = 0;
+
+    if (wine_rb_put( &manager->kernel_objects, &user_ptr, &kernel_object->rb_entry ))
+    {
+        /* kernel_object pointer already set */
+        free( kernel_object );
+        return NULL;
+    }
+
+    list_add_head( list, &kernel_object->list_entry );
+    return kernel_object;
+}
+
+static struct kernel_object *kernel_object_from_ptr( struct device_manager *manager, client_ptr_t client_ptr )
+{
+    struct wine_rb_entry *entry = wine_rb_get( &manager->kernel_objects, &client_ptr );
+    return entry ? WINE_RB_ENTRY_VALUE( entry, struct kernel_object, rb_entry ) : NULL;
+}
+
+static void grab_kernel_object( struct kernel_object *ptr )
+{
+    if (!ptr->owned)
+    {
+        grab_object( ptr->object );
+        ptr->owned = 1;
+    }
+}
+
 static void irp_call_dump( struct object *obj, int verbose )
 {
     struct irp_call *irp = (struct irp_call *)obj;
@@ -248,7 +338,7 @@ static struct irp_call *create_irp( struct device_file *file, const irp_params_t
 {
     struct irp_call *irp;
 
-    if (!file->device->manager)  /* it has been deleted */
+    if (file && !file->device->manager)  /* it has been deleted */
     {
         set_error( STATUS_FILE_DELETED );
         return NULL;
@@ -256,7 +346,7 @@ static struct irp_call *create_irp( struct device_file *file, const irp_params_t
 
     if ((irp = alloc_object( &irp_call_ops )))
     {
-        irp->file     = (struct device_file *)grab_object( file );
+        irp->file     = file ? (struct device_file *)grab_object( file ) : NULL;
         irp->thread   = NULL;
         irp->async    = NULL;
         irp->params   = *params;
@@ -325,15 +415,11 @@ static void device_destroy( struct object *obj )
     if (device->manager) list_remove( &device->entry );
 }
 
-static void add_irp_to_queue( struct device_file *file, struct irp_call *irp, struct thread *thread )
+static void add_irp_to_queue( struct device_manager *manager, struct irp_call *irp, struct thread *thread )
 {
-    struct device_manager *manager = file->device->manager;
-
-    assert( manager );
-
     grab_object( irp );  /* grab reference for queued irp */
     irp->thread = thread ? (struct thread *)grab_object( thread ) : NULL;
-    list_add_tail( &file->requests, &irp->dev_entry );
+    if (irp->file) list_add_tail( &irp->file->requests, &irp->dev_entry );
     list_add_tail( &manager->requests, &irp->mgr_entry );
     if (list_head( &manager->requests ) == &irp->mgr_entry) wake_up( &manager->obj, 0 );  /* first one */
 }
@@ -347,7 +433,8 @@ static struct object *device_open_file( struct object *obj, unsigned int access,
     if (!(file = alloc_object( &device_file_ops ))) return NULL;
 
     file->device = (struct device *)grab_object( device );
-    file->user_ptr = 0;
+    file->closed = 0;
+    list_init( &file->kernel_object );
     list_init( &file->requests );
     list_add_tail( &device->files, &file->entry );
     if (device->unix_path)
@@ -374,21 +461,27 @@ static struct object *device_open_file( struct object *obj, unsigned int access,
         irp_params_t params;
 
         memset( &params, 0, sizeof(params) );
-        params.create.major   = IRP_MJ_CREATE;
+        params.create.type    = IRP_CALL_CREATE;
         params.create.access  = access;
         params.create.sharing = sharing;
         params.create.options = options;
-        params.create.device  = file->device->user_ptr;
+        params.create.device  = get_kernel_object_ptr( device->manager, &device->obj );
 
         if ((irp = create_irp( file, &params, NULL )))
         {
-            add_irp_to_queue( file, irp, NULL );
+            add_irp_to_queue( device->manager, irp, NULL );
             release_object( irp );
         }
     }
     return &file->obj;
 }
 
+static struct list *device_get_kernel_obj_list( struct object *obj )
+{
+    struct device *device = (struct device *)obj;
+    return &device->kernel_object;
+}
+
 static void device_file_dump( struct object *obj, int verbose )
 {
     struct device_file *file = (struct device_file *)obj;
@@ -403,22 +496,28 @@ static struct fd *device_file_get_fd( struct object *obj )
     return (struct fd *)grab_object( file->fd );
 }
 
+static struct list *device_file_get_kernel_obj_list( struct object *obj )
+{
+    struct device_file *file = (struct device_file *)obj;
+    return &file->kernel_object;
+}
+
 static int device_file_close_handle( struct object *obj, struct process *process, obj_handle_t handle )
 {
     struct device_file *file = (struct device_file *)obj;
 
-    if (file->device->manager && obj->handle_count == 1)  /* last handle */
+    if (!file->closed && file->device->manager && obj->handle_count == 1)  /* last handle */
     {
         struct irp_call *irp;
         irp_params_t params;
 
+        file->closed = 1;
         memset( &params, 0, sizeof(params) );
-        params.close.major = IRP_MJ_CLOSE;
-        params.close.file  = file->user_ptr;
+        params.close.type = IRP_CALL_CLOSE;
 
         if ((irp = create_irp( file, &params, NULL )))
         {
-            add_irp_to_queue( file, irp, NULL );
+            add_irp_to_queue( file->device->manager, irp, NULL );
             release_object( irp );
         }
     }
@@ -440,26 +539,50 @@ static void device_file_destroy( struct object *obj )
     release_object( file->device );
 }
 
-static void set_file_user_ptr( struct device_file *file, client_ptr_t ptr )
+static int fill_irp_params( struct device_manager *manager, struct irp_call *irp, irp_params_t *params )
 {
-    struct irp_call *irp;
-
-    if (file->user_ptr == ptr) return;  /* nothing to do */
-
-    file->user_ptr = ptr;
-
-    /* update already queued irps */
-
-    LIST_FOR_EACH_ENTRY( irp, &file->requests, struct irp_call, dev_entry )
+    switch (irp->params.type)
     {
-        switch (irp->params.major)
-        {
-        case IRP_MJ_CLOSE:          irp->params.close.file = ptr; break;
-        case IRP_MJ_READ:           irp->params.read.file  = ptr; break;
-        case IRP_MJ_WRITE:          irp->params.write.file = ptr; break;
-        case IRP_MJ_FLUSH_BUFFERS:  irp->params.flush.file = ptr; break;
-        case IRP_MJ_DEVICE_CONTROL: irp->params.ioctl.file = ptr; break;
-        }
+    case IRP_CALL_NONE:
+    case IRP_CALL_FREE:
+        break;
+    case IRP_CALL_CREATE:
+        irp->params.create.file    = alloc_handle( current->process, irp->file,
+                                                   irp->params.create.access, 0 );
+        if (!irp->params.create.file) return 0;
+        break;
+    case IRP_CALL_CLOSE:
+        irp->params.close.file     = get_kernel_object_ptr( manager, &irp->file->obj );
+        break;
+    case IRP_CALL_READ:
+        irp->params.read.file      = get_kernel_object_ptr( manager, &irp->file->obj );
+        irp->params.read.out_size  = irp->iosb->out_size;
+        break;
+    case IRP_CALL_WRITE:
+        irp->params.write.file     = get_kernel_object_ptr( manager, &irp->file->obj );
+        break;
+    case IRP_CALL_FLUSH:
+        irp->params.flush.file     = get_kernel_object_ptr( manager, &irp->file->obj );
+        break;
+    case IRP_CALL_IOCTL:
+        irp->params.ioctl.file     = get_kernel_object_ptr( manager, &irp->file->obj );
+        irp->params.ioctl.out_size = irp->iosb->out_size;
+        break;
+    }
+
+    *params = irp->params;
+    return 1;
+}
+
+static void free_irp_params( struct irp_call *irp )
+{
+    switch (irp->params.type)
+    {
+    case IRP_CALL_CREATE:
+        close_handle( current->process, irp->params.create.file );
+        break;
+    default:
+        break;
     }
 }
 
@@ -471,7 +594,7 @@ static int queue_irp( struct device_file *file, const irp_params_t *params, stru
 
     fd_queue_async( file->fd, async, ASYNC_TYPE_WAIT );
     irp->async = (struct async *)grab_object( async );
-    add_irp_to_queue( file, irp, current );
+    add_irp_to_queue( file->device->manager, irp, current );
     release_object( irp );
     set_error( STATUS_PENDING );
     return 1;
@@ -488,10 +611,9 @@ static int device_file_read( struct fd *fd, struct async *async, file_pos_t pos
     irp_params_t params;
 
     memset( &params, 0, sizeof(params) );
-    params.read.major = IRP_MJ_READ;
-    params.read.key   = 0;
-    params.read.pos   = pos;
-    params.read.file  = file->user_ptr;
+    params.read.type = IRP_CALL_READ;
+    params.read.key  = 0;
+    params.read.pos  = pos;
     return queue_irp( file, &params, async );
 }
 
@@ -501,10 +623,9 @@ static int device_file_write( struct fd *fd, struct async *async, file_pos_t pos
     irp_params_t params;
 
     memset( &params, 0, sizeof(params) );
-    params.write.major = IRP_MJ_WRITE;
-    params.write.key   = 0;
-    params.write.pos   = pos;
-    params.write.file  = file->user_ptr;
+    params.write.type = IRP_CALL_WRITE;
+    params.write.key  = 0;
+    params.write.pos  = pos;
     return queue_irp( file, &params, async );
 }
 
@@ -514,8 +635,7 @@ static int device_file_flush( struct fd *fd, struct async *async )
     irp_params_t params;
 
     memset( &params, 0, sizeof(params) );
-    params.flush.major = IRP_MJ_FLUSH_BUFFERS;
-    params.flush.file  = file->user_ptr;
+    params.flush.type = IRP_CALL_FLUSH;
     return queue_irp( file, &params, async );
 }
 
@@ -525,27 +645,23 @@ static int device_file_ioctl( struct fd *fd, ioctl_code_t code, struct async *as
     irp_params_t params;
 
     memset( &params, 0, sizeof(params) );
-    params.ioctl.major = IRP_MJ_DEVICE_CONTROL;
-    params.ioctl.code  = code;
-    params.ioctl.file = file->user_ptr;
+    params.ioctl.type = IRP_CALL_IOCTL;
+    params.ioctl.code = code;
     return queue_irp( file, &params, async );
 }
 
 static struct device *create_device( struct object *root, const struct unicode_str *name,
-                                     struct device_manager *manager, unsigned int attr )
+                                     struct device_manager *manager )
 {
     struct device *device;
 
-    if ((device = create_named_object( root, &device_ops, name, attr, NULL )))
+    if ((device = create_named_object( root, &device_ops, name, 0, NULL )))
     {
-        if (get_error() != STATUS_OBJECT_NAME_EXISTS)
-        {
-            /* initialize it if it didn't already exist */
-            device->unix_path = NULL;
-            device->manager = manager;
-            list_add_tail( &manager->devices, &device->entry );
-            list_init( &device->files );
-        }
+        device->unix_path = NULL;
+        device->manager = manager;
+        list_add_tail( &manager->devices, &device->entry );
+        list_init( &device->kernel_object );
+        list_init( &device->files );
     }
     return device;
 }
@@ -559,6 +675,7 @@ struct object *create_unix_device( struct object *root, const struct unicode_str
     {
         device->unix_path = strdup( unix_path );
         device->manager = NULL;  /* no manager, requests go straight to the Unix device */
+        list_init( &device->kernel_object );
         list_init( &device->files );
     }
     return &device->obj;
@@ -608,13 +725,37 @@ static int device_manager_signaled( struct object *obj, struct wait_queue_entry
 static void device_manager_destroy( struct object *obj )
 {
     struct device_manager *manager = (struct device_manager *)obj;
+    struct kernel_object *kernel_object;
     struct list *ptr;
 
+    if (manager->current_call)
+    {
+        release_object( manager->current_call );
+        manager->current_call = NULL;
+    }
+
+    while (manager->kernel_objects.root)
+    {
+        kernel_object = WINE_RB_ENTRY_VALUE( manager->kernel_objects.root, struct kernel_object, rb_entry );
+        wine_rb_remove( &manager->kernel_objects, &kernel_object->rb_entry );
+        list_remove( &kernel_object->list_entry );
+        if (kernel_object->owned) release_object( kernel_object->object );
+        free( kernel_object );
+    }
+
     while ((ptr = list_head( &manager->devices )))
     {
         struct device *device = LIST_ENTRY( ptr, struct device, entry );
         delete_device( device );
     }
+
+    while ((ptr = list_head( &manager->requests )))
+    {
+        struct irp_call *irp = LIST_ENTRY( ptr, struct irp_call, mgr_entry );
+        list_remove( &irp->mgr_entry );
+        assert( !irp->file && !irp->async );
+        release_object( irp );
+    }
 }
 
 static struct device_manager *create_device_manager(void)
@@ -623,12 +764,44 @@ static struct device_manager *create_device_manager(void)
 
     if ((manager = alloc_object( &device_manager_ops )))
     {
+        manager->current_call = NULL;
         list_init( &manager->devices );
         list_init( &manager->requests );
+        wine_rb_init( &manager->kernel_objects, compare_kernel_object );
     }
     return manager;
 }
 
+void free_kernel_objects( struct object *obj )
+{
+    struct list *ptr, *list;
+
+    if (!(list = obj->ops->get_kernel_obj_list( obj ))) return;
+
+    while ((ptr = list_head( list )))
+    {
+        struct kernel_object *kernel_object = LIST_ENTRY( ptr, struct kernel_object, list_entry );
+        struct irp_call *irp;
+        irp_params_t params;
+
+        assert( !kernel_object->owned );
+
+        memset( &params, 0, sizeof(params) );
+        params.free.type = IRP_CALL_FREE;
+        params.free.obj  = kernel_object->user_ptr;
+
+        if ((irp = create_irp( NULL, &params, NULL )))
+        {
+            add_irp_to_queue( kernel_object->manager, irp, NULL );
+            release_object( irp );
+        }
+
+        list_remove( &kernel_object->list_entry );
+        wine_rb_remove( &kernel_object->manager->kernel_objects, &kernel_object->rb_entry );
+        free( kernel_object );
+    }
+}
+
 
 /* create a device manager */
 DECL_HANDLER(create_device_manager)
@@ -661,10 +834,13 @@ DECL_HANDLER(create_device)
         return;
     }
 
-    if ((device = create_device( root, &name, manager, req->attributes )))
+    if ((device = create_device( root, &name, manager )))
     {
-        device->user_ptr = req->user_ptr;
-        reply->handle = alloc_handle( current->process, device, req->access, req->attributes );
+        struct kernel_object *ptr = set_kernel_object( manager, &device->obj, req->user_ptr );
+        if (ptr)
+            grab_kernel_object( ptr );
+        else
+            set_error( STATUS_NO_MEMORY );
         release_object( device );
     }
 
@@ -676,13 +852,23 @@ DECL_HANDLER(create_device)
 /* delete a device */
 DECL_HANDLER(delete_device)
 {
+    struct device_manager *manager;
+    struct kernel_object *ref;
     struct device *device;
 
-    if ((device = (struct device *)get_handle_obj( current->process, req->handle, 0, &device_ops )))
+    if (!(manager = (struct device_manager *)get_handle_obj( current->process, req->manager,
+                                                             0, &device_manager_ops )))
+        return;
+
+    if ((ref = kernel_object_from_ptr( manager, req->device )) && ref->object->ops == &device_ops)
     {
+        device = (struct device *)grab_object( ref->object );
         delete_device( device );
         release_object( device );
     }
+    else set_error( STATUS_INVALID_HANDLE );
+
+    release_object( manager );
 }
 
 
@@ -694,8 +880,6 @@ DECL_HANDLER(get_next_device_request)
     struct list *ptr;
     struct iosb *iosb;
 
-    reply->params.major = IRP_MJ_MAXIMUM_FUNCTION + 1;
-
     if (!(manager = (struct device_manager *)get_handle_obj( current->process, req->manager,
                                                              0, &device_manager_ops )))
         return;
@@ -708,29 +892,42 @@ DECL_HANDLER(get_next_device_request)
             close_handle( current->process, req->prev );  /* avoid an extra round-trip for close */
             release_object( irp );
         }
-        clear_error();
     }
 
+    if (manager->current_call)
+    {
+        free_irp_params( manager->current_call );
+        release_object( manager->current_call );
+        manager->current_call = NULL;
+    }
+
+    clear_error();
+
     if ((ptr = list_head( &manager->requests )))
     {
         irp = LIST_ENTRY( ptr, struct irp_call, mgr_entry );
         if (irp->thread)
         {
-            reply->client_pid = get_process_id( irp->thread->process );
-            reply->client_tid = get_thread_id( irp->thread );
+            reply->client_thread = get_kernel_object_ptr( manager, &irp->thread->obj );
+            reply->client_tid    = get_thread_id( irp->thread );
         }
-        reply->params = irp->params;
         iosb = irp->iosb;
         reply->in_size = iosb->in_size;
-        reply->out_size = iosb->out_size;
         if (iosb->in_size > get_reply_max_size()) set_error( STATUS_BUFFER_OVERFLOW );
-        else if ((reply->next = alloc_handle( current->process, irp, 0, 0 )))
+        else if (!irp->file || (reply->next = alloc_handle( current->process, irp, 0, 0 )))
         {
-            set_reply_data_ptr( iosb->in_data, iosb->in_size );
-            iosb->in_data = NULL;
-            iosb->in_size = 0;
-            list_remove( &irp->mgr_entry );
-            list_init( &irp->mgr_entry );
+            if (fill_irp_params( manager, irp, &reply->params ))
+            {
+                set_reply_data_ptr( iosb->in_data, iosb->in_size );
+                iosb->in_data = NULL;
+                iosb->in_size = 0;
+                list_remove( &irp->mgr_entry );
+                list_init( &irp->mgr_entry );
+                /* we already own the object if it's only on manager queue */
+                if (irp->file) grab_object( irp );
+                manager->current_call = irp;
+            }
+            else close_handle( current->process, reply->next );
         }
     }
     else set_error( STATUS_PENDING );
@@ -746,9 +943,111 @@ DECL_HANDLER(set_irp_result)
 
     if ((irp = (struct irp_call *)get_handle_obj( current->process, req->handle, 0, &irp_call_ops )))
     {
-        if (irp->file) set_file_user_ptr( irp->file, req->file_ptr );
         set_irp_result( irp, req->status, get_req_data(), get_req_data_size(), req->size );
         close_handle( current->process, req->handle );  /* avoid an extra round-trip for close */
         release_object( irp );
     }
 }
+
+
+/* get kernel pointer from server object */
+DECL_HANDLER(get_kernel_object_ptr)
+{
+    struct device_manager *manager;
+    struct object *object = NULL;
+
+    if (!(manager = (struct device_manager *)get_handle_obj( current->process, req->manager,
+                                                             0, &device_manager_ops )))
+        return;
+
+    if ((object = get_handle_obj( current->process, req->handle, 0, NULL )))
+    {
+        reply->user_ptr = get_kernel_object_ptr( manager, object );
+        release_object( object );
+    }
+
+    release_object( manager );
+}
+
+
+/* associate kernel pointer with server object */
+DECL_HANDLER(set_kernel_object_ptr)
+{
+    struct device_manager *manager;
+    struct object *object = NULL;
+
+    if (!(manager = (struct device_manager *)get_handle_obj( current->process, req->manager,
+                                                             0, &device_manager_ops )))
+        return;
+
+    if (!(object = get_handle_obj( current->process, req->handle, 0, NULL )))
+    {
+        release_object( manager );
+        return;
+    }
+
+    if (!set_kernel_object( manager, object, req->user_ptr ))
+        set_error( STATUS_INVALID_HANDLE );
+
+    release_object( object );
+    release_object( manager );
+}
+
+
+/* grab server object reference from kernel object pointer */
+DECL_HANDLER(grab_kernel_object)
+{
+    struct device_manager *manager;
+    struct kernel_object *ref;
+
+    if (!(manager = (struct device_manager *)get_handle_obj( current->process, req->manager,
+                                                             0, &device_manager_ops )))
+        return;
+
+    if ((ref = kernel_object_from_ptr( manager, req->user_ptr )) && !ref->owned)
+        grab_kernel_object( ref );
+    else
+        set_error( STATUS_INVALID_HANDLE );
+
+    release_object( manager );
+}
+
+
+/* release server object reference from kernel object pointer */
+DECL_HANDLER(release_kernel_object)
+{
+    struct device_manager *manager;
+    struct kernel_object *ref;
+
+    if (!(manager = (struct device_manager *)get_handle_obj( current->process, req->manager,
+                                                             0, &device_manager_ops )))
+        return;
+
+    if ((ref = kernel_object_from_ptr( manager, req->user_ptr )) && ref->owned)
+    {
+        ref->owned = 0;
+        release_object( ref->object );
+    }
+    else set_error( STATUS_INVALID_HANDLE );
+
+    release_object( manager );
+}
+
+
+/* get handle from kernel object pointer */
+DECL_HANDLER(get_kernel_object_handle)
+{
+    struct device_manager *manager;
+    struct kernel_object *ref;
+
+    if (!(manager = (struct device_manager *)get_handle_obj( current->process, req->manager,
+                                                             0, &device_manager_ops )))
+        return;
+
+    if ((ref = kernel_object_from_ptr( manager, req->user_ptr )))
+        reply->handle = alloc_handle( current->process, ref->object, req->access, 0 );
+    else
+        set_error( STATUS_INVALID_HANDLE );
+
+    release_object( manager );
+}
