@@ -164,6 +164,7 @@ static const struct
     { "armv5",   CPU_ARM },
     { "armv6",   CPU_ARM },
     { "armv7",   CPU_ARM },
+    { "armv7a",  CPU_ARM },
     { "arm64",   CPU_ARM64 },
     { "aarch64", CPU_ARM64 },
 };
@@ -204,14 +205,17 @@ struct options
     int compile_only;
     int force_pointer_size;
     int large_address_aware;
+    int wine_builtin;
     int unwind_tables;
     int strip;
+    int pic;
     const char* wine_objdir;
     const char* output_name;
     const char* image_base;
     const char* section_align;
     const char* lib_suffix;
     const char* subsystem;
+    const char* prelink;
     strarray* prefix;
     strarray* lib_dirs;
     strarray* linker_args;
@@ -377,9 +381,17 @@ static int try_link( const strarray *prefix, const strarray *link_tool, const ch
     return ret;
 }
 
-static const strarray* get_lddllflags( const struct options *opts, const strarray *link_tool )
+static strarray *get_link_args( struct options *opts, const char *output_name )
 {
+    const strarray *link_tool = get_translator( opts );
     strarray *flags = strarray_alloc();
+    unsigned int i;
+
+    strarray_addall( flags, link_tool );
+    for (i = 0; i < opts->linker_args->size; i++) strarray_add( flags, opts->linker_args->base[i] );
+
+    if (verbose > 1) strarray_add( flags, "-v" );
+
     switch (opts->target_platform)
     {
     case PLATFORM_APPLE:
@@ -391,34 +403,94 @@ static const strarray* get_lddllflags( const struct options *opts, const strarra
             strarray_add( flags, "-read_only_relocs" );
             strarray_add( flags, "warning" );
         }
+        if (opts->image_base)
+        {
+            strarray_add( flags, "-image_base" );
+            strarray_add( flags, opts->image_base );
+        }
+        if (opts->strip) strarray_add( flags, "-Wl,-x" );
+        return flags;
+
+    case PLATFORM_SOLARIS:
+        {
+            char *mapfile = get_temp_file( output_name, ".map" );
+            const char *align = opts->section_align ? opts->section_align : "0x1000";
+
+            create_file( mapfile, 0644, "text = A%s;\ndata = A%s;\n", align, align );
+            strarray_add( flags, strmake("-Wl,-M,%s", mapfile) );
+            strarray_add( tmp_files, mapfile );
+        }
         break;
 
     case PLATFORM_ANDROID:
-    case PLATFORM_SOLARIS:
-    case PLATFORM_UNSPECIFIED:
-        strarray_add( flags, "-shared" );
-        strarray_add( flags, "-Wl,-Bsymbolic" );
-
-        /* Try all options first - this is likely to succeed on modern compilers */
-        if (!try_link( opts->prefix, link_tool, "-fPIC -shared -Wl,-Bsymbolic "
-                       "-Wl,-z,defs -Wl,-init,__wine_spec_init,-fini,_wine_spec_fini" ))
-        {
-            strarray_add( flags, "-Wl,-z,defs" );
-            strarray_add( flags, "-Wl,-init,__wine_spec_init,-fini,__wine_spec_fini" );
-        }
-        else /* otherwise figure out which ones are allowed */
-        {
-            if (!try_link( opts->prefix, link_tool, "-fPIC -shared -Wl,-Bsymbolic -Wl,-z,defs" ))
-                strarray_add( flags, "-Wl,-z,defs" );
-            if (!try_link( opts->prefix, link_tool, "-fPIC -shared -Wl,-Bsymbolic "
-                           "-Wl,-init,__wine_spec_init,-fini,_wine_spec_fini" ))
-                strarray_add( flags, "-Wl,-init,__wine_spec_init,-fini,__wine_spec_fini" );
-        }
+        /* the Android loader requires a soname for all libraries */
+        strarray_add( flags, strmake( "-Wl,-soname,%s.so", output_name ));
         break;
 
+    case PLATFORM_WINDOWS:
+    case PLATFORM_CYGWIN:
+        if (opts->shared)
+        {
+            strarray_add( flags, "-shared" );
+            strarray_add( flags, "-Wl,--kill-at" );
+        }
+        else strarray_add( flags, opts->gui_app ? "-mwindows" : "-mconsole" );
+
+        if (opts->unicode_app) strarray_add( flags, "-municode" );
+        if (opts->nodefaultlibs) strarray_add( flags, "-nodefaultlibs" );
+        if (opts->nostartfiles) strarray_add( flags, "-nostartfiles" );
+
+        if (opts->subsystem)
+        {
+            strarray_add( flags, strmake("-Wl,--subsystem,%s", opts->subsystem ));
+            if (!strcmp( opts->subsystem, "native" ))
+            {
+                const char *entry = opts->target_cpu == CPU_x86 ? "_DriverEntry@8" : "DriverEntry";
+                strarray_add( flags, strmake( "-Wl,--entry,%s", entry ));
+            }
+        }
+
+        if (opts->image_base) strarray_add( flags, strmake("-Wl,--image-base,%s", opts->image_base ));
+
+        if (opts->large_address_aware && opts->target_cpu == CPU_x86)
+            strarray_add( flags, "-Wl,--large-address-aware" );
+
+        return flags;
+
     default:
-        assert(0);
+        if (opts->image_base)
+        {
+            if (!try_link( opts->prefix, link_tool, strmake("-Wl,-Ttext-segment=%s", opts->image_base)) )
+                strarray_add( flags, strmake("-Wl,-Ttext-segment=%s", opts->image_base) );
+            else
+                opts->prelink = PRELINK;
+        }
+        if (!try_link( opts->prefix, link_tool, "-Wl,-z,max-page-size=0x1000"))
+            strarray_add( flags, "-Wl,-z,max-page-size=0x1000");
+        break;
     }
+
+    /* generic Unix shared library flags */
+
+    strarray_add( flags, "-shared" );
+    strarray_add( flags, "-Wl,-Bsymbolic" );
+
+    /* Try all options first - this is likely to succeed on modern compilers */
+    if (!try_link( opts->prefix, link_tool, "-fPIC -shared -Wl,-Bsymbolic "
+                   "-Wl,-z,defs -Wl,-init,__wine_spec_init,-fini,_wine_spec_fini" ))
+    {
+        strarray_add( flags, "-Wl,-z,defs" );
+        strarray_add( flags, "-Wl,-init,__wine_spec_init,-fini,__wine_spec_fini" );
+    }
+    else /* otherwise figure out which ones are allowed */
+    {
+        if (!try_link( opts->prefix, link_tool, "-fPIC -shared -Wl,-Bsymbolic -Wl,-z,defs" ))
+            strarray_add( flags, "-Wl,-z,defs" );
+        if (!try_link( opts->prefix, link_tool, "-fPIC -shared -Wl,-Bsymbolic "
+                       "-Wl,-init,__wine_spec_init,-fini,_wine_spec_fini" ))
+            strarray_add( flags, "-Wl,-init,__wine_spec_init,-fini,__wine_spec_fini" );
+    }
+
     return flags;
 }
 
@@ -445,6 +517,28 @@ static int check_platform( struct options *opts, const char *file )
     return ret;
 }
 
+static void make_wine_builtin( const char *file )
+{
+    static const char wine_magic[32] = "Wine builtin DLL";
+    int fd;
+    struct
+    {
+        unsigned short e_magic;
+        unsigned short unused[29];
+        unsigned int   e_lfanew;
+    } header;
+
+    if ((fd = open( file, O_RDWR | O_BINARY )) == -1) error( "Failed to add signature to %s\n", file );
+
+    if (read( fd, &header, sizeof(header) ) == sizeof(header) && !memcmp( &header.e_magic, "MZ", 2 ))
+    {
+        if (header.e_lfanew < sizeof(header) + sizeof(wine_magic))
+            error( "Not enough space (%x) for Wine signature\n", header.e_lfanew );
+        write( fd, wine_magic, sizeof(wine_magic) );
+    }
+    close( fd );
+}
+
 static const char *get_multiarch_dir( enum target_cpu cpu )
 {
    switch(cpu)
@@ -591,7 +685,10 @@ static void compile(struct options* opts, const char* lang)
             strarray_add(comp_args, "-DWINE_UNICODE_NATIVE");
 	}
         strarray_add(comp_args, "-D_REENTRANT");
-        strarray_add(comp_args, "-fPIC");
+        if (opts->pic)
+            strarray_add(comp_args, "-fPIC");
+        else
+            strarray_add(comp_args, "-fno-PIC");
     }
 
     if (opts->target_cpu == CPU_x86_64 || opts->target_cpu == CPU_ARM64)
@@ -686,7 +783,7 @@ no_compat_defines:
     /* last, but not least, the files */
     for ( j = 0; j < opts->files->size; j++ )
     {
-	if (opts->files->base[j][0] != '-' || !opts->files->base[j][1]) /* not an option or bare '-' (i.e. stdin) */
+	if (opts->files->base[j][0] != '-')
 	    strarray_add(comp_args, opts->files->base[j]);
     }
 
@@ -748,21 +845,6 @@ static strarray *get_winebuild_args(struct options *opts)
     return spec_args;
 }
 
-static const char* compile_resources_to_object(struct options* opts, const strarray *resources,
-                                               const char *res_o_name)
-{
-    strarray *winebuild_args = get_winebuild_args( opts );
-
-    strarray_add( winebuild_args, "--resources" );
-    strarray_add( winebuild_args, "-o" );
-    strarray_add( winebuild_args, res_o_name );
-    strarray_addall( winebuild_args, resources );
-
-    spawn( opts->prefix, winebuild_args, 0 );
-    strarray_free( winebuild_args );
-    return res_o_name;
-}
-
 /* check if there is a static lib associated to a given dll */
 static char *find_static_lib( const char *dll )
 {
@@ -799,37 +881,16 @@ static void add_library( struct options *opts, strarray *lib_dirs, strarray *fil
     free(fullname);
 }
 
-/* hack a main or WinMain function to work around Mingw's lack of Unicode support */
-static const char *mingw_unicode_hack( struct options *opts )
-{
-    char *main_stub = get_temp_file( opts->output_name, ".c" );
-
-    create_file( main_stub, 0644,
-                 "typedef unsigned short wchar_t;\n"
-                 "extern void * __stdcall LoadLibraryA(const char *);\n"
-                 "extern void * __stdcall GetProcAddress(void *,const char *);\n"
-                 "extern int wmain( int argc, wchar_t *argv[] );\n\n"
-                 "int main( int argc, char *argv[] )\n{\n"
-                 "    int wargc;\n"
-                 "    wchar_t **wargv, **wenv;\n"
-                 "    void *msvcrt = LoadLibraryA( \"msvcrt.dll\" );\n"
-                 "    void (*__wgetmainargs)(int *argc, wchar_t** *wargv, wchar_t** *wenvp, int expand_wildcards,\n"
-                 "                           int *new_mode) = GetProcAddress( msvcrt, \"__wgetmainargs\" );\n"
-                 "    __wgetmainargs( &wargc, &wargv, &wenv, 0, 0 );\n"
-                 "    return wmain( wargc, wargv );\n}\n" );
-    return compile_to_object( opts, main_stub, NULL );
-}
-
 static void build(struct options* opts)
 {
     strarray *lib_dirs, *files;
     strarray *spec_args, *link_args;
-    char *output_file;
+    char *output_file, *output_path;
     const char *spec_o_name;
     const char *output_name, *spec_file, *lang;
-    const char *prelink = NULL;
     int generate_app_loader = 1;
     int fake_module = 0;
+    int is_pe = (opts->target_platform == PLATFORM_WINDOWS || opts->target_platform == PLATFORM_CYGWIN);
     unsigned int j;
 
     /* NOTE: for the files array we'll use the following convention:
@@ -852,7 +913,7 @@ static void build(struct options* opts)
     }
 
     /* generate app loader only for .exe */
-    if (opts->shared || strendswith(output_file, ".so"))
+    if (opts->shared || is_pe || strendswith(output_file, ".so"))
 	generate_app_loader = 0;
 
     if (strendswith(output_file, ".fake")) fake_module = 1;
@@ -864,6 +925,7 @@ static void build(struct options* opts)
     else output_name = output_file;
     if (!strchr(output_name, '.'))
         output_file = strmake("%s.%s", output_file, opts->shared ? "dll" : "exe");
+    output_path = is_pe ? output_file : strmake( "%s.so", output_file );
 
     /* get the filename from the path */
     if ((output_name = strrchr(output_file, '/'))) output_name++;
@@ -888,7 +950,6 @@ static void build(struct options* opts)
     /* mark the files with their appropriate type */
     spec_file = lang = 0;
     files = strarray_alloc();
-    link_args = strarray_alloc();
     for ( j = 0; j < opts->files->size; j++ )
     {
 	const char* file = opts->files->base[j];
@@ -933,150 +994,14 @@ static void build(struct options* opts)
 	    lang = file;
     }
 
-    /* building for Windows is completely different */
-
-    if (opts->target_platform == PLATFORM_WINDOWS || opts->target_platform == PLATFORM_CYGWIN)
-    {
-        strarray *resources = strarray_alloc();
-        char *res_o_name = NULL;
-
-        if (opts->win16_app)
-            error( "Building 16-bit code is not supported for Windows\n" );
-
-        strarray_addall(link_args, get_translator(opts));
-
-        if (opts->shared)
-        {
-            /* run winebuild to generate the .def file */
-            char *spec_def_name = get_temp_file(output_name, ".spec.def");
-            spec_args = get_winebuild_args( opts );
-            strarray_add(spec_args, "--def");
-            strarray_add(spec_args, "-o");
-            strarray_add(spec_args, spec_def_name);
-            if (spec_file)
-            {
-                strarray_add(spec_args, "--export");
-                strarray_add(spec_args, spec_file);
-            }
-            spawn(opts->prefix, spec_args, 0);
-            strarray_free(spec_args);
-
-            strarray_add(link_args, "-shared");
-            if (verbose) strarray_add(link_args, "-v");
-            strarray_add(link_args, "-Wl,--kill-at");
-            strarray_add(link_args, spec_def_name);
-        }
-        else
-        {
-            strarray_add(link_args, opts->gui_app ? "-mwindows" : "-mconsole");
-        }
-
-        if (opts->nodefaultlibs) strarray_add(link_args, "-nodefaultlibs");
-        if (opts->nostartfiles) strarray_add(link_args, "-nostartfiles" );
-
-        if (opts->subsystem)
-        {
-            strarray_add(link_args, strmake("-Wl,--subsystem,%s", opts->subsystem));
-            if (!strcmp( opts->subsystem, "native" ))
-            {
-                const char *entry = opts->target_cpu == CPU_x86 ? "_DriverEntry@8" : "DriverEntry";
-                strarray_add(link_args, strmake( "-Wl,--entry,%s", entry ));
-            }
-        }
-
-        for ( j = 0 ; j < opts->linker_args->size ; j++ )
-            strarray_add(link_args, opts->linker_args->base[j]);
-
-        strarray_add(link_args, "-o");
-        strarray_add(link_args, output_file);
-
-        if (opts->image_base)
-            strarray_add(link_args, strmake("-Wl,--image-base,%s", opts->image_base));
-
-        if (opts->large_address_aware && opts->target_cpu == CPU_x86)
-            strarray_add( link_args, "-Wl,--large-address-aware" );
-
-        if (opts->unicode_app && !opts->shared)
-            strarray_add(link_args, mingw_unicode_hack(opts));
-
-        for ( j = 0; j < lib_dirs->size; j++ )
-            strarray_add(link_args, strmake("-L%s", lib_dirs->base[j]));
-
-        if (!opts->nodefaultlibs)
-        {
-            add_library(opts, lib_dirs, files, "winecrt0");
-            add_library(opts, lib_dirs, files, "kernel32");
-            add_library(opts, lib_dirs, files, "ntdll");
-        }
-        if (!opts->shared && opts->use_msvcrt && opts->target_platform == PLATFORM_CYGWIN)
-            add_library(opts, lib_dirs, files, "msvcrt");
-
-        for ( j = 0; j < files->size; j++ )
-        {
-            const char* name = files->base[j] + 2;
-
-            switch(files->base[j][1])
-            {
-            case 'l':
-            case 'd':
-                strarray_add(link_args, strmake("-l%s", name));
-                break;
-            case 's':
-            case 'o':
-                strarray_add(link_args, name);
-                break;
-            case 'a':
-                if (!opts->lib_suffix && strchr(name, '/'))
-                {
-                    /* turn the path back into -Ldir -lfoo options
-                     * this makes sure that we use the specified libs even
-                     * when mingw adds its own import libs to the link */
-                    char *lib = xstrdup( name );
-                    char *p = strrchr( lib, '/' );
-
-                    *p++ = 0;
-                    if (!strncmp( p, "lib", 3 ))
-                    {
-                        char *ext = strrchr( p, '.' );
-
-                        if (ext) *ext = 0;
-                        p += 3;
-                        strarray_add(link_args, strmake("-L%s", lib ));
-                        strarray_add(link_args, strmake("-l%s", p ));
-                        free( lib );
-                        break;
-                    }
-                    free( lib );
-                }
-                strarray_add(link_args, name);
-                break;
-            case 'r':
-                if (!res_o_name)
-                {
-                    res_o_name = get_temp_file( output_name, ".res.o" );
-                    strarray_add( link_args, res_o_name );
-                }
-                strarray_add( resources, name );
-                break;
-            }
-        }
-
-        if (res_o_name) compile_resources_to_object( opts, resources, res_o_name );
-
-        spawn(opts->prefix, link_args, 0);
-        strarray_free (resources);
-        strarray_free (link_args);
-        strarray_free (lib_dirs);
-        strarray_free (files);
-        return;
-    }
+    if (opts->win16_app && is_pe)
+        error( "Building 16-bit code is not supported for Windows\n" );
 
     /* add the default libraries, if needed */
-    if (!opts->nostdlib && opts->use_msvcrt) add_library(opts, lib_dirs, files, "msvcrt");
 
-    if (!opts->wine_objdir && !opts->nodefaultlibs) 
+    if (!opts->wine_objdir && !opts->nodefaultlibs)
     {
-        if (opts->gui_app) 
+        if (opts->gui_app)
 	{
 	    add_library(opts, lib_dirs, files, "shell32");
 	    add_library(opts, lib_dirs, files, "comdlg32");
@@ -1088,12 +1013,13 @@ static void build(struct options* opts)
 
     if (!opts->nodefaultlibs)
     {
+        if (opts->use_msvcrt) add_library(opts, lib_dirs, files, "msvcrt");
         add_library(opts, lib_dirs, files, "winecrt0");
         if (opts->win16_app) add_library(opts, lib_dirs, files, "kernel");
         add_library(opts, lib_dirs, files, "kernel32");
         add_library(opts, lib_dirs, files, "ntdll");
     }
-    if (!opts->nostdlib) add_library(opts, lib_dirs, files, "wine");
+    if (!opts->nostdlib && !is_pe) add_library(opts, lib_dirs, files, "wine");
 
     /* run winebuild to generate the .spec.o file */
     spec_args = get_winebuild_args( opts );
@@ -1104,7 +1030,7 @@ static void build(struct options* opts)
     if (opts->force_pointer_size)
         strarray_add(spec_args, strmake("-m%u", 8 * opts->force_pointer_size ));
     strarray_add(spec_args, "-D_REENTRANT");
-    strarray_add(spec_args, "-fPIC");
+    if (opts->pic && !is_pe) strarray_add(spec_args, "-fPIC");
     strarray_add(spec_args, opts->shared ? "--dll" : "--exe");
     if (fake_module)
     {
@@ -1173,52 +1099,10 @@ static void build(struct options* opts)
     if (fake_module) return;  /* nothing else to do */
 
     /* link everything together now */
-    strarray_addall(link_args, get_translator(opts));
-    strarray_addall(link_args, get_lddllflags(opts, link_args));
+    link_args = get_link_args( opts, output_name );
 
     strarray_add(link_args, "-o");
-    strarray_add(link_args, strmake("%s.so", output_file));
-
-    for ( j = 0 ; j < opts->linker_args->size ; j++ ) 
-        strarray_add(link_args, opts->linker_args->base[j]);
-
-    switch (opts->target_platform)
-    {
-    case PLATFORM_APPLE:
-        if (opts->image_base)
-        {
-            strarray_add(link_args, "-image_base");
-            strarray_add(link_args, opts->image_base);
-        }
-        if (opts->strip)
-            strarray_add(link_args, "-Wl,-x");
-        break;
-    case PLATFORM_SOLARIS:
-        {
-            char *mapfile = get_temp_file( output_name, ".map" );
-            const char *align = opts->section_align ? opts->section_align : "0x1000";
-
-            create_file( mapfile, 0644, "text = A%s;\ndata = A%s;\n", align, align );
-            strarray_add(link_args, strmake("-Wl,-M,%s", mapfile));
-            strarray_add(tmp_files, mapfile);
-        }
-        break;
-    case PLATFORM_ANDROID:
-        /* the Android loader requires a soname for all libraries */
-        strarray_add( link_args, strmake( "-Wl,-soname,%s.so", output_name ));
-        break;
-    default:
-        if (opts->image_base)
-        {
-            if (!try_link(opts->prefix, link_args, strmake("-Wl,-Ttext-segment=%s", opts->image_base)))
-                strarray_add(link_args, strmake("-Wl,-Ttext-segment=%s", opts->image_base));
-            else
-                prelink = PRELINK;
-        }
-        if (!try_link(opts->prefix, link_args, "-Wl,-z,max-page-size=0x1000"))
-            strarray_add(link_args, "-Wl,-z,max-page-size=0x1000");
-        break;
-    }
+    strarray_add(link_args, output_path);
 
     for ( j = 0; j < lib_dirs->size; j++ )
 	strarray_add(link_args, strmake("-L%s", lib_dirs->base[j]));
@@ -1234,14 +1118,38 @@ static void build(struct options* opts)
 		strarray_add(link_args, strmake("-l%s", name));
 		break;
 	    case 's':
-	    case 'a':
 	    case 'o':
 		strarray_add(link_args, name);
 		break;
+	    case 'a':
+                if (is_pe && !opts->lib_suffix && strchr(name, '/'))
+                {
+                    /* turn the path back into -Ldir -lfoo options
+                     * this makes sure that we use the specified libs even
+                     * when mingw adds its own import libs to the link */
+                    char *lib = xstrdup( name );
+                    char *p = strrchr( lib, '/' );
+
+                    *p++ = 0;
+                    if (!strncmp( p, "lib", 3 ) && strcmp( p, "libmsvcrt.a" ))
+                    {
+                        char *ext = strrchr( p, '.' );
+
+                        if (ext) *ext = 0;
+                        p += 3;
+                        strarray_add(link_args, strmake("-L%s", lib ));
+                        strarray_add(link_args, strmake("-l%s", p ));
+                        free( lib );
+                        break;
+                    }
+                    free( lib );
+                }
+		strarray_add(link_args, name);
+		break;
 	}
     }
 
-    if (!opts->nostdlib) 
+    if (!opts->nostdlib && !is_pe)
     {
 	strarray_add(link_args, "-lm");
 	strarray_add(link_args, "-lc");
@@ -1250,16 +1158,18 @@ static void build(struct options* opts)
     spawn(opts->prefix, link_args, 0);
     strarray_free (link_args);
 
+    if (is_pe && opts->wine_builtin) make_wine_builtin( output_path );
+
     /* set the base address with prelink if linker support is not present */
-    if (prelink && !opts->target)
+    if (opts->prelink && !opts->target)
     {
-        if (prelink[0] && strcmp(prelink,"false"))
+        if (opts->prelink[0] && strcmp(opts->prelink,"false"))
         {
             strarray *prelink_args = strarray_alloc();
-            strarray_add(prelink_args, prelink);
+            strarray_add(prelink_args, opts->prelink);
             strarray_add(prelink_args, "--reloc-only");
             strarray_add(prelink_args, opts->image_base);
-            strarray_add(prelink_args, strmake("%s.so", output_file));
+            strarray_add(prelink_args, output_path);
             spawn(opts->prefix, prelink_args, 1);
             strarray_free(prelink_args);
         }
@@ -1439,6 +1349,7 @@ int main(int argc, char **argv)
     opts.linker_args = strarray_alloc();
     opts.compiler_args = strarray_alloc();
     opts.winebuild_args = strarray_alloc();
+    opts.pic = 1;
 
     /* determine the processor type */
     if (strendswith(argv[0], "winecpp")) opts.processor = proc_cpp;
@@ -1447,7 +1358,7 @@ int main(int argc, char **argv)
     /* parse options */
     for ( i = 1 ; i < argc ; i++ ) 
     {
-        if (argv[i][0] == '-' && argv[i][1])  /* option, except '-' alone is stdin, which is a file */
+        if (argv[i][0] == '-')  /* option */
 	{
 	    /* determine if this switch is followed by a separate argument */
 	    next_is_arg = 0;
@@ -1560,6 +1471,10 @@ int main(int argc, char **argv)
                         opts.unwind_tables = 1;
 		    else if (!strcmp("-fno-asynchronous-unwind-tables", argv[i]))
                         opts.unwind_tables = 0;
+                    else if (!strcmp("-fPIC", argv[i]) || !strcmp("-fpic", argv[i]))
+                        opts.pic = 1;
+                    else if (!strcmp("-fno-PIC", argv[i]) || !strcmp("-fno-pic", argv[i]))
+                        opts.pic = 0;
 		    break;
 		case 'l':
 		    strarray_add(opts.files, strmake("-l%s", option_arg));
@@ -1668,6 +1583,11 @@ int main(int argc, char **argv)
                                 opts.large_address_aware = 1;
                                 continue;
                             }
+                            if (!strcmp(Wl->base[j], "--wine-builtin"))
+                            {
+                                opts.wine_builtin = 1;
+                                continue;
+                            }
                             if (!strcmp(Wl->base[j], "--subsystem") && j < Wl->size - 1)
                             {
                                 opts.subsystem = strdup( Wl->base[++j] );
